/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2016 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/


package com.adobe.cq.msm.ui.models.alllivecopies;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Pattern;

import javax.annotation.PostConstruct;
import javax.jcr.RangeIterator;

import org.apache.commons.lang3.StringUtils;
import org.apache.jackrabbit.util.Text;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.commons.json.JSONArray;
import org.apache.sling.commons.json.JSONException;
import org.apache.sling.commons.json.io.JSONStringer;
import org.apache.sling.models.annotations.Model;
import org.apache.sling.models.annotations.injectorspecific.OSGiService;
import org.apache.sling.models.annotations.injectorspecific.ScriptVariable;
import org.apache.sling.models.annotations.injectorspecific.Self;

import com.adobe.cq.wcm.launches.utils.LaunchUtils;
import com.adobe.granite.ui.components.Config;
import com.adobe.granite.ui.components.ExpressionHelper;
import com.adobe.granite.ui.components.ExpressionResolver;
import com.adobe.granite.xss.XSSAPI;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.wcm.api.Page;
import com.day.cq.wcm.api.PageManager;
import com.day.cq.wcm.api.WCMException;
import com.day.cq.wcm.msm.api.BlueprintManager;
import com.day.cq.wcm.msm.api.LiveRelationship;
import com.day.cq.wcm.msm.api.LiveRelationshipManager;
import com.day.cq.wcm.msm.api.LiveStatus;
import com.day.cq.wcm.msm.api.MSMNameConstants;

@Model(adaptables = SlingHttpServletRequest.class)
public class LiveCopiesTable {

    @Self
    private SlingHttpServletRequest request;

    private Resource resource;
    private Resource rootResource;
    private ResourceResolver resourceResolver;
    private List<LiveCopiesRow> rows;


    private PageManager pageManager;

    private LiveRelationshipManager relationMgr;

    private BlueprintManager bpm;


    private Map<String, String> tableAttrbsMap;

    private boolean tableExists;


    private Map<String, String> liveCopiesMap;

    private Pattern excludePattern;

    @ScriptVariable
    private XSSAPI xssAPI;

    @OSGiService
    private ExpressionResolver expressionResolver;

    private static HashMap<String, StatusDetails> statusDetailsMap = new HashMap<String, StatusDetails>();

    private static final String MSM_STATUS_STNCHRONIZED= "msm-status-synchronized";

    private static final String MSM_STATUS_SOURCE_MODIFIED = "msm-status-src-modified";

    private static final String MSM_STATUS_TARGET_MODIFIED = "msm-status-target-modified";

    private static final String MSM_STATUS_NEW = "msm-status-new";

    private static final String MSM_STATUS_TARGET_DOES_NOT_EXIST = "msm-status-target-does-not-exist";

    private static final String MSM_STATUS_CANCELLED = "msm-status-cancelled";

    private static final String MSM_BLUEPRINT_EDIT_PAGE_ACTION = "cq-wcm-msm-all-live-copies-openpage-activator";

    private static final String MSM_BLUEPRINT_ROLLOUT_ACTION = "cq-wcm-msm-all-live-copies-rollout-activator";

    private static final String MSM_LIVECOPY_EDIT_PAGE_ACTION = "cq-wcm-msm-all-live-copies-openlivecopy-activator";

    private static final String MSM_LIVECOPY_RELATIONSHIP_ACTION = "cq-wcm-msm-all-live-copies-relationship-activator";

    private static final String MSM_LIVECOPY_SYNC_ACTION = "cq-wcm-msm-all-live-copies-sync-activator";

    private static final String MSM_LIVECOPY_RESET_ACTION = "cq-wcm-msm-all-live-copies-reset-activator";

    private static final String MSM_LIVECOPY_SUSPEND_ACTION = "cq-wcm-msm-all-live-copies-suspend-activator";

    private static final String MSM_LIVECOPY_RESUME_ACTION = "cq-wcm-msm-all-live-copies-resume-activator";

    private static final String MSM_LIVECOPY_DETACH_ACTION = "cq-wcm-msm-all-live-copies-detach-activator";

    static {
        // for localization
        // i18n.get("LIVE COPY UP TO DATE");
        // i18n.get("BLUEPRINT MODIFIED");
        // i18n.get("LIVE COPY MODIFIED")
        // i18n.get("LIVE COPY LOCALLY CREATED");
        // i18n.get("LIVE COPY DOES NOT EXIST");
        // i18n.get("INHERITANCE CANCELLED");
        statusDetailsMap.put(MSM_STATUS_STNCHRONIZED, new StatusDetails("checkCircle", "cq-icon-green", "LIVE COPY UP TO DATE"));
        statusDetailsMap.put(MSM_STATUS_SOURCE_MODIFIED, new StatusDetails("clock", "cq-icon-black", "BLUEPRINT MODIFIED"));
        statusDetailsMap.put(MSM_STATUS_TARGET_MODIFIED, new StatusDetails("clock", "cq-icon-black", "LIVE COPY MODIFIED"));
        statusDetailsMap.put(MSM_STATUS_NEW, new StatusDetails("addCircle", "cq-icon-black", "LIVE COPY LOCALLY CREATED"));
        statusDetailsMap.put(MSM_STATUS_TARGET_DOES_NOT_EXIST, new StatusDetails("exclude", "cq-icon-grey", "LIVE COPY DOES NOT EXIST"));
        statusDetailsMap.put(MSM_STATUS_CANCELLED, new StatusDetails("closeCircle", "cq-icon-red", "INHERITANCE CANCELLED"));

    }


    @PostConstruct
    public void postConstruct() throws Exception {
	}

    private boolean isResourceAcceptable(Resource currResource) {
		return false;
	}

    private boolean isResourceBlueprint(Resource currResource) throws WCMException {
		return false;
	}

    private String getBlueprintActionRels(Resource currResource, boolean isBlueprint) {
		return null;
	}

    private String getLivecopyActionRels(String statusMesg) throws JSONException {
		return null;
	}





    private String getThumbnailUrl(Page page, int width, int height) {
		return null;
	}

    public String getPath() {
		return null;
	}

    public Map<String, String> getColumns() {
		return null;
	}

    public List<LiveCopiesRow> getRows() {
		return null;
	}

    public Map<String, String> getTableAttrbs() {
		return null;
	}

    public boolean tableExist() {
		return false;
	}

    public boolean hasChildren() {
		return false;
	}

    public int columnsSize() {
		return 0;
	}


    private String getRelativeSourcePath(Resource rootResource, Resource currResource) {
		return null;
	}
    private Map<String,String> getFilteredNames(RangeIterator rootRelations) {
		return null;
	}

    private String getStatus(LiveRelationshipManager relationMgr, Resource source, String targetFilter) throws Exception {
		return null;
	}



    private boolean getAdvanceStatusValue(Map<String, Boolean> advanceStatus, String param) {
		return false;
	}
    
    
    private static class StatusDetails {
        
        private String iconName;
        private String iconClass;
        private String statusMessage;
        
        private StatusDetails (String iconName, String iconClass, String statusMessage) {
            this.iconClass = iconClass;
            this.iconName = iconName;
            this.statusMessage = statusMessage;
            
        }
        
        public String getIconName() {
			return null;
		}
        
        public String getIconClass() {
			return null;
		}
        
        public String getStatusMessage() {
			return null;
		}
     }



}
