/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.comments.api;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Locale;
import java.util.Map.Entry;

import javax.jcr.RepositoryException;
import javax.jcr.Session;


import com.adobe.cq.social.commons.CommentException;
import com.adobe.cq.social.commons.comments.impl.CommentImpl;
import com.adobe.cq.social.scf.core.BaseQueryRequestInfo;
import com.adobe.cq.social.commons.comments.impl.TransientPropertyConstants;
import com.adobe.cq.social.commons.comments.impl.TransientPropertyHelper;
import com.day.cq.wcm.api.Page;
import org.apache.commons.lang3.StringUtils;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.SyntheticResource;
import org.apache.sling.api.resource.ValueMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.Attachment;
import com.adobe.cq.social.commons.CommentSystem;
import com.adobe.cq.social.commons.CommentUtil;
import com.adobe.cq.social.commons.comments.states.internal.State;
import com.adobe.cq.social.commons.comments.states.internal.StateMachine;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentList;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentListProvider;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentListProviderManager;
import com.adobe.cq.social.commons.moderation.api.FlagReason;
import com.adobe.cq.social.commons.tagging.SocialTagManager;
import com.adobe.cq.social.scf.ClientUtilities;
import com.adobe.cq.social.scf.CollectionPagination;
import com.adobe.cq.social.scf.QueryRequestInfo;
import com.adobe.cq.social.scf.SocialComponent;
import com.adobe.cq.social.scf.SocialComponentFactory;
import com.adobe.cq.social.scf.SocialComponentFactoryManager;
import com.adobe.cq.social.scf.User;
import com.adobe.cq.social.scf.core.BaseSocialComponent;
import com.adobe.cq.social.scf.core.CollectionSortedOrder;
import com.adobe.cq.social.srp.SocialResource;
import com.adobe.cq.social.srp.SocialResourceProvider;
import com.adobe.cq.social.srp.utilities.internal.InternalSocialResourceUtilities;
import com.adobe.cq.social.tally.Voting;
import com.adobe.cq.social.tally.client.api.Response;
import com.adobe.cq.social.tally.client.api.TallyException;
import com.adobe.cq.social.tally.client.api.Vote;
import com.adobe.cq.social.translation.TranslationResults;
import com.adobe.cq.social.translation.TranslationSCFUtil;
import com.adobe.cq.social.translation.TranslationUtil;
import com.adobe.cq.social.ugcbase.CollabUser;
import com.adobe.cq.social.ugcbase.SocialUtils;
import com.adobe.cq.social.ugcbase.core.SocialResourceUtils;
import com.adobe.granite.security.user.UserProperties;
import com.day.cq.tagging.TagConstants;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;

public abstract class AbstractComment<T extends CommentCollectionConfiguration> extends BaseSocialComponent implements
    Comment<T> {
    /** Logger for this class. */
    private static final Logger LOG = LoggerFactory.getLogger(AbstractComment.class);

    private static final String UNKNOWN_USER = "Unknown";
    private static final String JCR_DESCRIPTION_PROP = "jcr:description";
    private static final String JCR_TITLE_PROP = "jcr:title";
    private static String USERS_HOME_UGC = "/home/users/";

    /** local variables. */
    protected final CommentSocialComponentList replies;
    private final User author;
    private final Map<String, Attachment> attachments;
    private final Calendar created;
    private final String message;
    private final String parentPath;
    private final String resourceType;
    private final boolean isTopLevel;
    private final boolean isApproved;
    private boolean isDraft = false;
    private final State state;
    private boolean isScheduled = false;
    private final Calendar publishDate;
    private boolean canEdit = false;
    private boolean canDelete = false;
    private boolean userIsModerator = false;
    private boolean currentUserFlagged = false;
    private String currentUserFlagText = null;
    private String translationAttribution = null;
    private final QueryRequestInfo queryInfo;
    private final boolean isVisible;
    private final boolean isClosed;
    private final boolean isPinned;
    private final boolean isFeatured;
    private final boolean doDisplayTranslation;
    private final boolean displayReplyButton;
    private T configuration;
    private final ModeratorActions moderatorActions;
    private final ModeratorStatus moderatorStatus;
    private final String votingRoot;
    private boolean useFlagReasonList = true;
    private boolean useFlagReasons = false;
    private boolean useReferrerUrl;
    private List<FlagReason> flagReasons;
    private CommentSocialComponentListProviderManager commentListProviderManager;
    private final com.adobe.cq.social.commons.Comment comment;
    private SocialComponent parentComponent;
    private SocialComponent sourceComponent;
    private PageInfo pageInfo;
    private String descriptionTranslation = null;
    private String titleTranslation = null;
    private String displayTranslation = null;
    private int MAX_ERROR_MESSAGE_LENGTH = 1000; // max message length that we log in the case filterHTML failing.

    private final String MODERATORS_ROLE = "moderators";
    private final String OWNER_ROLE = "owner";
    private final String EDIT_OPERATION = "Edit";
    private final String DELETE_OPERATION = "Delete";
    private final String PROP_STATE = "state_s";
    private final String ERROR_HTML_FILTER_MESSAGE =
        "There was something wrong with this post.  Please contact your administrator to fix the issue.";
    private final String FILTER = "filter";
    private final String PAGE = "page has";
    private final String RESOURCE_COMMENTS = "se_social/se_discussion/";
    private final String CATALOG = "catalog.";
    private final String LEARNINGPATH = "learningpath";
    private final String RESOURCE = "resource";

    private String filterFromRequest = null;

    /**
     * Construct a comment for the specified resource and client utilities.
     * @param resource the specified resource
     * @param clientUtils the client utilities instance
     * @param commentListProviderManager list manager to use for listing content
     * @throws RepositoryException if an error occurs
     */
    public AbstractComment(final Resource resource, final ClientUtilities clientUtils,
        final CommentSocialComponentListProviderManager commentListProviderManager) throws RepositoryException {
        this(resource, clientUtils, QueryRequestInfo.DEFAULT_QUERY_INFO_FACTORY.create(), commentListProviderManager);
    }

    /**
     * Constructor of a comment.
     * @param resource the specified {@link com.adobe.cq.social.commons.Comment}
     * @param clientUtils the client utilities instance
     * @param queryInfo the query info.
     * @param commentListProviderManager list manager to use for listing content
     * @throws RepositoryException if an error occurs
     */
    public AbstractComment(final Resource resource, final ClientUtilities clientUtils,
        final QueryRequestInfo queryInfo, final CommentSocialComponentListProviderManager commentListProviderManager)
        throws RepositoryException {
        super(resource, clientUtils);
        this.commentListProviderManager = commentListProviderManager;
        comment = resource.adaptTo(com.adobe.cq.social.commons.Comment.class);
        author = this.clientUtils.getUser(comment.getAuthor().getId(), comment.getResource().getResourceResolver());
        attachments = comment.getAttachments();
        created = comment.getCreated();
        message = comment.getMessage();
        parentPath = comment.getComponent().getPath();
        isTopLevel = comment.isTopLevel();

        //This is needed for deeplinking of repliess
        if (queryInfo.getPredicates().containsKey(FILTER) && (queryInfo.getPredicates().get(FILTER)[0].startsWith(PAGE))) {
            filterFromRequest = queryInfo.getPredicates().get(FILTER)[0].replace(PAGE, "").trim();
            this.queryInfo = new BaseQueryRequestInfo(false, new HashMap<String, String[]>(), queryInfo.getPagination(), queryInfo.getSortOrder(), queryInfo.getSortBy());
        } else {
            this.queryInfo = queryInfo;
        }

        flagReasons = new ArrayList<FlagReason>();

        final Session session = resource.getResourceResolver().adaptTo(Session.class);
        final String userId = session.getUserID();

        final CommentSystem cs = comment.getCommentSystem();
        // Comment is approved or it doesn't need to be because it's not moderated.
        isApproved = !comment.isSpam() && (comment.isApproved() || !cs.isModerated());
        userIsModerator = clientUtils.getSocialUtils().hasModeratePermissions(resource);
        isDraft = comment.getProperty(com.adobe.cq.social.commons.Comment.PROP_IS_DRAFT, false);
        // to get the state of the comment
        // TODO: adaptor factory
        final String currentState = comment.getProperty(PROP_STATE, "");
        final StateMachine stateMachine = resource.adaptTo(StateMachine.class);
        state = stateMachine == null ? null : stateMachine.getState(currentState);
        isScheduled = comment.getProperty(com.adobe.cq.social.commons.Comment.PROP_IS_SCHEDULED, false);
        publishDate = comment.getProperty(com.adobe.cq.social.commons.Comment.PROP_PUBLISH_DATE, Calendar.class);
        final boolean userIsLoggedIn =
            clientUtils == null ? !(userId == null || userId.equalsIgnoreCase(CollabUser.ANONYMOUS)) : !clientUtils
                .userIsAnonymous();

        isClosed = comment.isClosed();
        isPinned = comment.getProperty(com.adobe.cq.social.commons.Comment.PROP_PINNED, false);
        isFeatured = comment.getProperty(com.adobe.cq.social.commons.Comment.PROP_FEATURED, false);
        useReferrerUrl = cs.getProperty(PROP_USE_REFERRER_URL, Boolean.FALSE);
        configuration = createConfiguration(resource, cs.getResource());
        // TODO: the complete logic should be inside state machine
        final boolean canModEdit = state == null ? true : state.canPerformOperation(MODERATORS_ROLE, EDIT_OPERATION);
        final boolean canOwnerEdit = state == null ? true : state.canPerformOperation(OWNER_ROLE, EDIT_OPERATION);
        final boolean canModDelete =
            state == null ? true : state.canPerformOperation(MODERATORS_ROLE, DELETE_OPERATION);
        final boolean canOwnerDelete = state == null ? true : state.canPerformOperation(OWNER_ROLE, DELETE_OPERATION);
        if ((userIsModerator && canModEdit) || (userIsOwner() && userIsLoggedIn && canOwnerEdit) && !isClosed) {
            canEdit = true;
        }

        if ((userIsModerator && canModDelete)
                || (userIsOwner() && userIsLoggedIn && canOwnerDelete && cs.allowsDelete()) && !isClosed) {
            canDelete = true;
        }

        // A comment is visible to moderators, and to others if it's not spam, not hidden from flagging,
        // and not premoderated and not yet approved.
        if (userIsModerator || (!comment.isSpam() && !comment.isFlaggedHidden() && isApproved && !isDraft)) {
            isVisible = true;
        } else {
            isVisible = false;
        }
        displayReplyButton = canUserReply(userIsLoggedIn, cs, session);

        moderatorStatus = (userIsModerator) ? new ModeratorStatusImpl(comment, cs) : null;

        if (cs.allowsFlagging()) {
            useFlagReasonList = cs.useFlagReasonList();
            useFlagReasons = useFlagReasonList || cs.allowCustomFlagReason();
            final Response<Vote> response = getFlagResponseForUser(userId);
            if (response != null && response.getResponseValue() != null) { // this user DID flag this content
                currentUserFlagged = true;
                if (useFlagReasons) {
                    final Resource responseResource = response.getResource();
                    final ValueMap resourceProperties = responseResource.adaptTo(ValueMap.class);

                    // Should not be null, but ensure it's not anyways.
                    currentUserFlagText =
                        resourceProperties.get(com.adobe.cq.social.commons.Comment.PROP_FLAG_REASON, "");
                }
            }

            if (userIsModerator && useFlagReasons) {
                flagReasons = listFlagReasons(resource, clientUtils);
            }
        }

        this.moderatorActions = new ModeratorActionsImpl(comment, cs, userIsLoggedIn);

        resourceType = comment.getResource().getResourceType();

        final QueryRequestInfo repliesListInfo = QueryRequestInfo.DEFAULT_QUERY_INFO_FACTORY.create(this.queryInfo);
        final CollectionPagination currentPagination = repliesListInfo.getPagination();
        CollectionPagination repliesPagination;

        if (configuration.isDoNotGetRepliesOnListingPage() && resource != null && TransientPropertyHelper.getTransientProperty(resource,TransientPropertyConstants.TRANSIENT_PROPERTY_COLLECTION, false)) {
            repliesPagination = new CollectionPagination(0, 0, 0, currentPagination.getSortIndex(), this.configuration.getPageSize());
        }else if (currentPagination.getEmbedLevel() != CollectionPagination.DEFAULT_EMBED_LEVEL) {
            // don't fetch nested replies
            repliesPagination =
                new CollectionPagination(0, 0, currentPagination.getEmbedLevel() + 1,
                    currentPagination.getSortIndex(), this.configuration.getPageSize());
        } else {
            repliesPagination =
                new CollectionPagination(currentPagination.getOffset(),
                    queryInfo.getPagination() == CollectionPagination.DEFAULT_PAGINATION ? this.configuration
                        .getPageSize() : currentPagination.getSize(),
                    currentPagination.getEmbedLevel() + 1, currentPagination.getSortIndex(),
                    this.configuration.getPageSize());
        }
        repliesListInfo.setPagination(repliesPagination);
        repliesListInfo.setSortOrder(CollectionSortedOrder.DEFAULT_ORDER);

        replies = getReplies(resource, this, this.filterFromRequest, repliesListInfo);

        final Resource component = cs.getResource();
        if (configuration.isVotingAllowed()) // set voting root only if its allowed for a SCF component
            votingRoot = CommentUtil.getVotingRoot(component);
        else
            votingRoot = "";

        // if this is a search, prefetches are done in SearchComponent and no need to show translate button
        if (!queryInfo.isQuery()
                && resource.getPath().indexOf(
                    clientUtils.getSocialUtils().getStorageConfig(resource).getAsiPath() + USERS_HOME_UGC) == -1) {
            prefetchResources(resource.getResourceResolver(), clientUtils, cs);

            // call the function from TranslationUtil to determine weather show the translation button.
            doDisplayTranslation =
                TranslationUtil.doDisplayTranslation(resource.getResourceResolver(), resource, clientUtils);
        } else {
            doDisplayTranslation = false;
        }

        pageInfo = new PageInfo(this, clientUtils, repliesListInfo.getPagination());

        if ((this.queryInfo.isTranslationRequest() && this.queryInfo.getPagination().getEmbedLevel() == 0)
                || (clientUtils.getRequest() != null && TranslationSCFUtil.isSmartRenderingOn(resource, clientUtils))) {
            final TranslationResults results = TranslationSCFUtil.getTranslationSCF(resource, clientUtils);
            if (results != null) {
                final Map<String, String> translations = results.getTranslation();
                this.displayTranslation = results.getDisplay();

                this.translationAttribution = results.getAttribution();
                if (!translations.isEmpty()) {
                    this.descriptionTranslation = translations.get(JCR_DESCRIPTION_PROP);
                    this.titleTranslation = translations.get(JCR_TITLE_PROP);
                }
            }
        }
    }

    /* Returns the index of a reply within all replies of a given comment for given conditions */
    private int getChildIndex(final String commentToCheck, final int size, final List<Entry<String, Boolean>> sortFields) throws CommentException {
		return 0;
	}

    /* This function just handles first level reply to a top level post. We will have to write separate function for nested levels */
    private CommentSocialComponentList getReplies(final Resource resource, final AbstractComment abstractComment, final String filterFromRequest, final QueryRequestInfo repliesListInfo) {
		return null;
	}

    /**
     * @param userIsLoggedIn - true id the current request is from a logged in user
     * @param cs - the {@link CommentSystem} that this {@link Comment} belongs to
     * @param session - the {@link Session} associated with the current resolver
     * @return true if the user can reply, false otherwise
     */
    protected boolean canUserReply(final boolean userIsLoggedIn, final CommentSystem cs, final Session session) {
		return false;
	}

    private void prefetchResources(final ResourceResolver resolver, final ClientUtilities clientUtils,
        final CommentSystem cs) {
		}

    /**
     * Obtain the configuration for this comment.
     * @param resource The comment resource
     * @param commentSystem Resource
     * @return T type
     */
    protected T createConfiguration(final Resource resource, final Resource commentSystem) {
		return null;
	}

    @Override
    public T getConfiguration() {
		return null;
	}

    private boolean userIsOwner() {
		return false;
	}

    /**
     * Get the path for the current version of the flag tally. This may or may not exist.
     * @return String containing the path, or null if the resource has no returnable properties.
     */
    private String getFlagsPath() {
		return null;
	}

    /**
     * Return response of current user to this comment. If this comment was not flagged by this user (or was last
     * unflagged), return null.
     * @return Response
     * @throws RepositoryException
     */
    private Response<Vote> getFlagResponseForUser(final String currentUserId) throws RepositoryException {
		return null;
	}

    /**
     * Get a list of flag reasons.
     * @param resource The comment resource
     * @param clientUtils
     * @return The list of flag reasons.
     * @throws RepositoryException
     */
    private List<FlagReason> listFlagReasons(final Resource resource, final ClientUtilities clientUtils)
        throws RepositoryException {
			return null;
		}

    @Override
    protected List<String> getIgnoredProperties() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Attachment getAttachment(final String name) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public User getAuthor() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getState() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, Attachment> getAttachments() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Calendar getCreated() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getResourceType() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getMessage() {
		return null;
	}

    /**
     * Gets the parent path for the comment.
     * @return a string pointing to the externalized URL to the parent.
     */
    public String getParent() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
		return null;
	}

    /**
     * Set the collection list range. If this is not set, then the return list will start at offset 0 and the default
     * size
     * @param pagination detail information to use
     */
    @Override
    public void setPagination(final CollectionPagination pagination) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void setSortedOrder(final CollectionSortedOrder sortedOrder) {
	}

    public void setSortFields(final List<Entry<String, Boolean>> sortFields) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isTopLevel() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isApproved() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getCanEdit() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getCanReply() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getCanDelete() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public int getTotalSize() {
		return 0;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public List<Object> getItems() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getParentId() {
		return null;
	}

    @Override
    public String getSourceComponentId() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getCanTranslate() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isUserModerator() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isVisible() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isClosed() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPinned() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isFeatured() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isDraft() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isScheduled() {
		return false;
	}

    @Override
    public Calendar getPublishDate() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public List<FlagReason> getFlagReasons() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getCurrentUserFlagText() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getUseFlagReasons() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isFlaggedByUser() {
		return false;
	}

    private static class SyntheticVotingResource extends SyntheticResource implements SocialResource {

        private final SocialResource parent;

        public SyntheticVotingResource(final ResourceResolver resourceResolver, final SocialResource parentIn,
            final String path, final String resourceType) {
            super(resourceResolver, path, resourceType);
            parent = parentIn;
        }

        @Override
        public SocialResourceProvider getResourceProvider() {
			return null;
		}

        @Override
        public Resource getRootJCRNode() {
			return null;
		}

        @Override
        public boolean checkPermissions(final String permission) {
			return false;
		}

    }

    @Override
    public SocialComponent getVotes() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public ModeratorActions getModeratorActions() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    @JsonInclude(Include.NON_NULL)
    public ModeratorStatus getModeratorStatus() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public SocialComponent getParentComponent() {
		return null;
	}

    /**
     * The idea here is to allow us to search a hierarchy of nodes for both the parent and the source at the same
     * time. In this case if we stumble along the source component while looking for the parent we will set it as
     * well. If only the parent is request we won't look higher for the source component, but if we are looking for
     * the source component we can start at the parent ( maybe skipping some nodes ).
     * @param forceSourceComponent
     */
    private void setParentOrSourceComponent(final boolean forceSourceComponent) {
	}

    private SocialComponent getComponent(final Resource srcResource) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public SocialComponent getSourceComponent() {
		return null;
	}

    /**
     * Get the list of replies.
     * @return the children of this comment.
     * @throws RepositoryException is thrown if there an error occurs while fetching the data.
     */
    protected List<Comment> getComments() throws RepositoryException {
		return null;
	}

    /**
     * Get the pagination information for the comment.
     * @return the QueryRequestInfo pagination details
     */
    protected CollectionPagination getPagination() {
		return null;
	}

    /**
     * Get the query info comment.
     * @return the QueryRequestInfo details
     */
    protected QueryRequestInfo getQueryRequestInfo() {
		return null;
	}

    @Override
    public List<Tag> getTags() {
		return null;
	}

    /**
     * Method to find out the locale of the page where tags are being pulled for
     * If there is an explicit content page path available, use that else fallback on Default Locale which is always Locale.ENGLISH
     * @param currentResource
     * @return locale
     */
    private Locale getPageLocale(Resource currentResource){
		return null;
	}
    protected class ModeratorStatusImpl implements ModeratorStatus {
        private final boolean flagged;
        private final boolean approved;
        private final boolean spam;
        private final boolean pending;

        public ModeratorStatusImpl(final com.adobe.cq.social.commons.Comment comment, final CommentSystem cs) {
            flagged = comment.isFlagged();
            spam = comment.isSpam();
            approved = !spam && (comment.isApproved() || !cs.isModerated());
            pending = cs.isModerated() && !comment.isApproved() && !comment.isDenied();
        }

        @Override
        @JsonProperty("isFlagged")
        public boolean isFlagged() {
			return false;
		}

        @Override
        @JsonProperty("isApproved")
        public boolean isApproved() {
			return false;
		}

        @Override
        @JsonProperty("isSpam")
        public boolean isSpam() {
			return false;
		}

        @Override
        @JsonProperty("isPending")
        public boolean isPending() {
			return false;
		}
    }

    protected class ModeratorActionsImpl implements ModeratorActions {
        private boolean displayAllowButton = false;
        private boolean displayFlagButton = false;
        private boolean displayDenyButton = false;
        private boolean displayCloseButton = false;
        private boolean displayReviewButton = false;
        private List<String> nextStates = new ArrayList<String>();
        private boolean displayPinButton = false;
        private boolean canMove = false;
        private boolean displayFeaturedLink = false;
        private final String OPERATION_REVIEW_IDEA = "Review Idea";

        public ModeratorActionsImpl(final com.adobe.cq.social.commons.Comment comment, final CommentSystem cs,
            final boolean userIsLoggedIn) {
            final boolean commentApproved = comment.isApproved() || !cs.isModerated();

            // Logic explanation:
            // SHOW the Flag button only if all of these are true
            // The comment is not closed
            // The current user has not flagged it
            // The user is logged in
            // The comment is approved or the comment system is not premoderated
            // The user doesn't own the resource (and is not a mod, as they are considered the owner ??)
            // The comment is not spam
            // The comment has not been flagged enough times to be hidden
            // The comment system allows flagging.
            // HIDE the Flag button if any of them are not true.
            displayFlagButton =
                !isClosed && !currentUserFlagged && userIsLoggedIn && commentApproved && !userIsOwner()
                        && !comment.isSpam() && !comment.isFlaggedHidden() && cs.allowsFlagging();

            displayAllowButton =
                userIsModerator && !isClosed && (comment.isSpam() || comment.isFlagged() || !commentApproved);

            displayDenyButton = userIsModerator && !comment.isSpam() && cs.allowsDeny();
            // close or open button are both covered under displayCloseButton:
            displayPinButton = userIsModerator && isTopLevel && cs.allowsPin();

            // close or open button are both covered under displayCloseButton:
            displayCloseButton = userIsModerator && isTopLevel && cs.allowsClose();
            // show "move" option if user is a moderator and if the comment system is configured to allow moves and if
            // its not closed
            canMove = userIsModerator && isTopLevel && !comment.isClosed() && cs.allowsMove();

            displayFeaturedLink = userIsModerator && isTopLevel && configuration.isFeaturingContentAllowed();

            // change state buttons
            displayReviewButton =
                state == null ? false : userIsModerator && isTopLevel
                        && state.canPerformOperation(MODERATORS_ROLE, OPERATION_REVIEW_IDEA);

            if (userIsModerator && isTopLevel && state != null) {
                nextStates = state.getNextStatesForRole(MODERATORS_ROLE);
            }
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanDeny() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getUseFlagReasonList() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanAllow() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanFlag() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanClose() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanPin() {
			return false;
		}

        @Override
        public boolean getCanMove() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean getCanMarkFeatured() {
			return false;
		}

        /**
         * @return true if the Review action button should be displayed.
         */
        public boolean getCanReview() {
			return false;
		}

        /**
         * @return list of the states that moderators can move the resource to.
         */
        public List<String> getReviewStates() {
			return null;
		}

    }

    /**
     * Gets information about the pages for this collection. This can be used by the page block helper to easily
     * render various pagination UIs.
     * @return information about the pagination system
     */
    @Override
    public PageInfo getPageInfo() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getTranslationDescription() {
		return null;
	}

    /**
     * {@inheritDoc}

     */
    @Override
    public String getTranslationAttribution() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getTranslationTitle() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getTranslationDisplay() {
		return null;
	}

    @Override
    public String getFriendlyUrl() {
		return null;
	}

    //Till we find a better of removing pagination from referrer. Ideally, it should be handled in comment creation
    // logic, but for backward compatibility, we need to handle it here
    private String removePageinationFromReferer(final String referer, final String enablementType) {
		return null;
	}

    @Override
    public String getReferrerUrl() {
		return null;
	}
}
