/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2015 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.subscriptions.client.api;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.jackrabbit.util.Text;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ValueMap;

import com.adobe.cq.social.graph.client.api.Following;
import com.adobe.cq.social.scf.ClientUtilities;
import com.adobe.cq.social.scf.QueryRequestInfo;
import com.adobe.cq.social.scf.core.BaseSocialComponent;
import com.adobe.cq.social.subscriptions.api.Subscription;
import com.adobe.cq.social.subscriptions.api.SubscriptionManager;
import com.adobe.cq.social.subscriptions.api.SubscriptionType;
import com.adobe.cq.social.subscriptions.endpoint.SubscriptionCollectionOperations;
import com.adobe.cq.social.ugcbase.CollabUser;

/**
 * Base class of SCF SubscriptionCollection component. A subscription collection represents a relationships between a
 * user and a resource or another user. The value of the userId and the followedId is obtained in the following order:
 * If the request is a Post request, then the userId and the followedId are obtained from the request parameters. If
 * the request is a Get request, then:
 * <p>
 * 1. The userId is obtained from the resource {@link Following#PROP_USERID} property; If this property does not
 * exist, the userId is obtained from the request's session user id.
 * <p>
 * 2. The followedId is obtained from the resource {@link Following#PROP_FOLLOWEDID} property; If this property does
 * not exist, the userId is obtained from the resource name
 */
public class AbstractSubscriptionCollection extends BaseSocialComponent implements SubscriptionCollection {
    private final ResourceResolver resolver;
    private String userId;
    private String otherId;
    private static String subscriptionTypes[];
    private SortedMap<String, SubscriptionStatus> states;
    private final SubscriptionManager subscriptionManager;
    private boolean isSubscribed;
    private boolean isUserLoggedIn;
    private static final String PN_ALLOW_SUBSCRIPTIONS = "allowSubscriptions";
    private static final boolean PV_DEFAULT_ALLOW_SUBSCRIPTIONS = false;
    private static final String SUBSRCRIPTION_RELATIONSHIP_ID = SubscriptionType.SUBSCRIPTIONS.getRelationshipId();

    static {
        final List<String> list = new ArrayList<String>();
        for (final SubscriptionType type : SubscriptionType.values()) {
            list.add(type.getRelationshipId());
        }
        final String buf[] = new String[list.size()];
        subscriptionTypes = list.toArray(buf);
    }

    public AbstractSubscriptionCollection(final Resource resource, final ClientUtilities clientUtils,
        final QueryRequestInfo q, final SubscriptionManager subscriptionManager) {
        super(resource, clientUtils);
        initializeNodes(resource, clientUtils);
        resolver = resource.getResourceResolver();
        this.subscriptionManager = subscriptionManager;

    }

    /**
     * Construct a collection from the specified subscriptions
     * @param resource
     * @param clientUtils
     * @param q
     * @param subscriptions
     */
    public AbstractSubscriptionCollection(final Resource resource, final ClientUtilities clientUtils,
        final QueryRequestInfo q, final List<Subscription> subscriptions) {
        super(resource, clientUtils);
        initializeNodes(resource, clientUtils);
        resolver = resource.getResourceResolver();
        this.subscriptionManager = null;
        states = createStateList(subscriptions);
    }

    private void initializeNodes(final Resource resource, final ClientUtilities clientUtils) {
	}

    @Override
    public String getSubscribedId() {
		return null;
	}

    @Override
    public String getOwner() {
		return null;
	}

    @Override
    public String getPath() {
		return null;
	}

    @Override
    public String[] getTypes() {
		return null;
	}

    private boolean getAllowEmailSubscribing(final Resource resource, boolean lookDeep) {
		return false;
	}

    @Override
    public Map<String, SubscriptionStatus> getStates() {
		return null;
	}

    private SortedMap<String, SubscriptionStatus> createStateList(final List<Subscription> subscriptions) {
		return null;
	}

    @Override
    public boolean isSubscribed() {
		return false;
	}

    @Override
    public boolean isUserLoggedIn() {
		return false;
	}

    static class SubscriptionStatusImpl implements SubscriptionStatus {
        final private String type;
        final private boolean state;

        public SubscriptionStatusImpl(final String type, final boolean state) {
            this.type = type;
            this.state = state;
        }

        @Override
        public String getDisplayName() {
			return null;
		}

        @Override
        public boolean isSelected() {
			return false;
		}

    }
}
