/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.translation.api;

import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipInputStream;

import aQute.bnd.annotation.ProviderType;
import com.adobe.granite.comments.Comment;
import com.adobe.granite.comments.CommentCollection;

//
/** The Interface TranslationObject. */
@ProviderType
public interface TranslationObject {

    /**
     * Gets the title of the TranslationObject. Title is auto-populated from TranslationObject.
     * @return the title
     */
    String getTitle() {
		return null;
	}

    /**
     * Gets the path to the Translation Object being translated. The path returned is to where the target Translation
     * Object is stored in AEM CRX de. The path is derived by coping that of the the source Translation Object.
     * @return the content url
     */
    String getTranslationObjectTargetPath() {
		return null;
	}

    /**
     * Gets the path to the source Translation Object. The path returned is to the where the source Translation Object
     * is stored in AEM CRX de.
     * @return the source content url
     */
    String getTranslationObjectSourcePath() {
		return null;
	}

    /**
     * Gets the version name of the source Object. If this name is null, then latest version of source content is
     * used.
     * @return the version name
     */
    String getSourceVersion() {
		return null;
	}

    /**
     * Gets the mime type of TranslationObject.
     * @return the mime type
     */
    String getMimeType() {
		return null;
	}

    /**
     * Gets confirmation return from the external translation service provider that they have received the
     * TranslationObject for translation. Translation service provider returns a unique TranslationObjectId.
     * @return Translation Object Id
     */
    String getId() {
		return null;
	}

    /**
     * Gets the comment collection. This returns all the comments added to the TranslationObject.
     * @return the comment Collection
     */
    CommentCollection<Comment> getCommentCollection() {
		return null;
	}

    /**
     * Gets the supporting translation objects list for each relation. Each TranslationObject can have a list of
     * supporting translation objects which add reference to the TranslationObject.
     * @return the Map of supporting translation objects list, key contains the relation ship name like source,
     *         variant, reference and value of that key is List of TranslationObject
     */
    Map<String, List<TranslationObject>> getSupportingTranslationObjectsIterator() throws TranslationException {
		return null;
	}

    /**
     * Gets the supporting translation objects count.
     * @return the supporting translation objects count
     */
    @Deprecated
    int getSupportingTranslationObjectsCount() throws TranslationException {
		return 0;
	}

    /**
     * Gets the Translation Job metadata
     * @return the Translation Job metadata
     */
    TranslationMetadata getTranslationJobMetadata() {
		return null;
	}

    /**
     * @deprecated Use {@link #getTranslationObjectXMLInputStream()} instead. Gets XML input stream for the
     *             Translation Object to be translated.
     * @return the translation Object input stream
     * @throws TranslationException the translation exception
     */
    @Deprecated
    InputStream getTranslationObjectInputStream() throws TranslationException {
		return null;
	}

    /**
     * Gets XML input stream for the Translation Object to be translated.
     * @return The translation Object XML input stream
     * @throws TranslationException the translation exception
     */
    InputStream getTranslationObjectXMLInputStream() throws TranslationException {
		return null;
	}

    /**
     * Gets XLIFF input stream for the Translation Object to be translated.
     * @param xliffVersion Requested version of XLIFF
     * @return The translation Object XLIFF input stream
     * @throws TranslationException the translation exception
     */
    InputStream getTranslationObjectXLIFFInputStream(String xliffVersion) throws TranslationException {
		return null;
	}

    /**
     * Gets the translated Object input stream. Returns null in case TranslationObject was not translated.
     * @return the translated Object input stream
     */
    InputStream getTranslatedObjectInputStream() {
		return null;
	}

    /**
     * Gets a zip input stream containing the preview of Translation Object. This zip contains all the files required
     * for offline preview of the translation object.
     * @return Zip input stream, containing the preview of Translation Object.
     */
    ZipInputStream getTranslationObjectPreview() {
		return null;
	}
}