/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.granite.translation.api;

import java.util.List;

import com.adobe.granite.translation.api.TranslationConstants.TranslationMethod;

//
/**
 * The TranslationServiceFactory is responsible for creating TranslationService objects for a specific connector. It
 * also registers the connector with the TranslationManager using a unique identifier ("Name").
 */
public interface TranslationServiceFactory {

    /**
     * Creates a new TranslationService object specific to this Factory.
     * @param translationMethod Specify which type of Translation service to create based on Translation Method
     * @param cloudConfigPath The path to cloud config file.
     * @return A TranslationService instance for the given factory.
     * @throws TranslationException the translation exception
     */
    TranslationService createTranslationService(TranslationMethod translationMethod, String cloudConfigPath)
        throws TranslationException {
		return null;
	}

    /**
     * Get the supported translation methods.
     * @return List of Translation method of the service property indicating the particular translation method this
     *         factory provides, e.g. machine translation or human translation.
     */
    List<TranslationMethod> getSupportedTranslationMethods() {
		return null;
	}

    /**
     * Gets the service factory name.
     * @return Name of the service property indicating the particular implementation this factory provides, e.g.
     *         adobe.
     */
    String getServiceFactoryName() {
		return null;
	}

    /**
     * Gets the service Cloud Config Class.
     * @return Class of the service Cloud Config. This class name will be used to determine the cloud config for the
     *         Translation service.
     */
    Class<?> getServiceCloudConfigClass() {
		return null;
	}

    /**
     * Unique name of the service property indicating the particular implementation this factory provides, e.g.
     * microsoft.
     */
    static final String PROPERTY_TRANSLATION_FACTORY = "translationFactory";
}