/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.ui.components;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.jsp.PageContext;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.scripting.SlingBindings;
import org.apache.sling.scripting.jsp.util.JspSlingHttpServletResponseWrapper;

import com.adobe.granite.ui.components.ds.DataSource;
import com.adobe.granite.ui.components.ds.EmptyDataSource;
import com.adobe.granite.ui.components.impl.BaseComponentHelper;
import com.adobe.granite.ui.components.impl.SlingIncludeObjectFactory;
import com.adobe.granite.ui.components.rendercondition.RenderCondition;
import com.adobe.granite.ui.components.rendercondition.RenderConditionHelper;
import com.adobe.granite.ui.components.rendercondition.SimpleRenderCondition;
import com.adobe.granite.xss.XSSAPI;
import com.day.cq.i18n.I18n;

/**
 * A convenient helper for development of Granite UI component.
 */
public class ComponentHelper {
    private static final String DEFAULT_LAYOUT_RT = "granite/ui/components/foundation/layouts/container";
    private final BaseComponentHelper base;
    private RenderConditionHelper renderConditionHelper;
    private SlingIncludeObjectFactory dataFetcher;

    public ComponentHelper(PageContext pageContext) {
        SlingBindings bindings = (SlingBindings) pageContext.getRequest().getAttribute(SlingBindings.class.getName());
        base = new BaseComponentHelper(bindings.getSling(), bindings.getRequest(), new JspSlingHttpServletResponseWrapper(pageContext));
    }

    /**
     * Returns I18n appropriate for the current page.
     * @return I18n appropriate for the current page
     */
    public I18n getI18n() {
		return null;
	}

    /**
     * Returns XSSAPI based on the current request.
     * @return XSSAPI based on the current request
     */
    public XSSAPI getXss() {
		return null;
	}

    /**
     * Returns the config of current resource of the page.
     * @return the config of current resource of the page
     */
    public Config getConfig() {
		return null;
	}

    /**
     * Returns the values that is applicable for the current page.
     * @return the values that is applicable for the current page
     */
    public Value getValue() {
		return null;
	}

    /**
     * Returns the ExpressionHelper appropriate for the current page.
     * @return the ExpressionHelper appropriate for the current page
     */
    public ExpressionHelper getExpressionHelper() {
		return null;
	}

    /**
     * Returns the client state.
     * @return the client state
     */
    public State getState() {
		return null;
	}

    private RenderConditionHelper getRenderConditionHelper() {
		return null;
	}

    private SlingIncludeObjectFactory getSlingIncludeObjectFactory() {
		return null;
	}

    /**
     * Consumes the current available tag for current page. If the request
     * doesn't have the tag applicable to the page, a new tag is created.
     * <p>
     * There is a mechanism such that a tag can be passed to another page when
     * including that page. This method is intended as a way to consume the tag
     * passed by other page. Component developer can make use this method to get
     * the main tag of the component regardless if there is a tag passed by
     * other page or not.
     *
     * @return the tag
     * @see #include(Resource, Tag)
     * @see #include(Resource, String, Tag)
     */
    public Tag consumeTag() {
		return null;
	}

    /**
     * Consumes the current available layout resource for current page. This
     * method first attempts to return the layout resource from the options,
     * second it will attempt to return {@link Config#LAYOUT} child node,
     * otherwise <code>null</code> is returned.
     * <p>
     * This method is usually called by layout implementation to get the layout
     * resource that can be passed by the caller.
     *
     * @return the resource
     * @see #includeForLayout(Resource, Options)
     * @see #includeForLayout(Resource, Resource, Options)
     */
    public Resource consumeLayoutResource() {
		return null;
	}

    /**
     * The overload of {@link #populateCommonAttrs(AttrBuilder, Resource)} using the current request's resource as the source.
     *
     * @param attrs the attribute builder
     */
    public void populateCommonAttrs(AttrBuilder attrs) {
	}

    /**
     * Populates the common attributes to the given {@link AttrBuilder}.
     *
     * <p>Currently the following common properties and nodes are read and processed from the given resource:
     *
     * <table summary="the common attributes to the given AttrBuilder">
     * <thead>
     *   <tr>
     *     <th>Name</th>
     *     <th>Type</th>
     *     <th>Description</th>
     *   </tr>
     * </thead>
     * <tbody>
     *   <tr>
     *     <td>granite:id</td>
     *     <td>Property: StringEL</td>
     *     <td>The id attribute.</td>
     *   </tr>
     *   <tr>
     *     <td>granite:rel</td>
     *     <td>Property: StringEL</td>
     *     <td>The class attribute. This is used to indicate the semantic relationship of the component similar to <code>rel</code> attribute.</td>
     *   </tr>
     *   <tr>
     *     <td>granite:class</td>
     *     <td>Property: StringEL</td>
     *     <td>The class attribute.</td>
     *   </tr>
     *   <tr>
     *     <td>granite:title</td>
     *     <td>Property: String</td>
     *     <td>The title attribute. This property is i18nable.</td>
     *   </tr>
     *   <tr>
     *     <td>granite:hidden</td>
     *     <td>Property: Boolean</td>
     *     <td>The hidden attribute.</td>
     *   </tr>
     *   <tr>
     *     <td>granite:itemscope</td>
     *     <td>Property: Boolean</td>
     *     <td>The itemscope attribute.</td>
     *   </tr>
     *   <tr>
     *     <td>granite:itemtype</td>
     *     <td>Property: String</td>
     *     <td>The itemtype attribute.</td>
     *   </tr>
     *   <tr>
     *     <td>granite:itemprop</td>
     *     <td>Property: String</td>
     *     <td>The itemprop attribute.</td>
     *   </tr>
     *   <tr>
     *     <td>granite:data</td>
     *     <td>Node</td>
     *     <td>Each property of this node is converted into a data-* attribute.
     *     If the property value is an instance of a String, it will be interpreted as StringEL.
     *     The property having a prefixed name is ignored.</td>
     *   </tr>
     * </tbody>
     * </table>
     *
     * @param attrs The attribute builder to populate to
     * @param src The resource of the source of the config
     */
    public void populateCommonAttrs(AttrBuilder attrs, Resource src) {
	}

    /**
     * Returns the options passed from another page. If no options is passed,
     * empty options is returned.
     * <p>
     * There is a mechanism such that options can be passed to another page when
     * including that page.
     *
     * @return the options passed from another page. if no options is passed,
     *         empty options is returned.
     * @see #include(Resource, Options)
     * @see #include(Resource, String, Options)
     */
    public Options getOptions() {
		return null;
	}

    /**
     * Returns the layout config of current resource of the page. This method is
     * setting the default resource type of the layout.
     *
     * @return the layout config of current resource of the page
     * @see LayoutBuilder#getResourceType()
     */
    public LayoutBuilder getLayout() {
		return null;
	}

    /**
     * Returns the associated resource type of current resource for the purpose rendering read only version.
     * First the granite:readOnlyResourceType property of the resource type of the current resource (the RT) is used.
     * Otherwise it is defaulted to <code>readonly</code> child resource of the RT.
     *
     * @return the associated resource type of current resource
     */
    public String getReadOnlyResourceType() {
		return null;
	}

    /**
     * Returns the associated resource type of the given content resource for the purpose rendering read only version.
     * First the granite:readOnlyResourceType property of the resource type of the content resource (the RT) is used.
     * Otherwise it is defaulted to <code>readonly</code> child resource of the RT.
     *
     * @param resource the resource
     * @return the associated resource type of the given content resource
     */
    public String getReadOnlyResourceType(Resource resource) {
		return null;
	}

    /**
     * Returns the datasource for items of the current resource. This is an
     * overload of {@link #getItemDataSource(Resource)} with resource is the
     * current request resource.
     *
     * @return the data source for items of the current resource
     * @throws ServletException in case there's a servlet error while fetching data
     * @throws IOException in case there's an i/o error while fetching data
     */
    public DataSource getItemDataSource() throws ServletException, IOException {
		return null;
	}

    /**
     * Returns the datasource for items of the given resource.
     * This method can be used to fetch the items that are specified literally
     * using {@link Config#ITEMS} subresource; or specified as datasource using {@link Config#DATASOURCE} subresource.
     *
     * If there is no {@link Config#ITEMS} or {@link Config#DATASOURCE} subresource, then {@link EmptyDataSource} is returned.
     *
     * In contrast with {@link #asDataSource(Resource, Resource)}, this method
     * looks for the datasource resource of the given resource. i.e. The given
     * resource is the parent of the items, not the datasource resource itself.
     * The given resource is also used as the context resource when calling
     * {@link #asDataSource(Resource, Resource)} internally.
     *
     * @param resource the resource
     * @return the data source for items of the given resource
     * @throws ServletException in case there's a servlet error while fetching data
     * @throws IOException in case there's an i/o error while fetching data
     */
    public DataSource getItemDataSource(Resource resource) throws ServletException, IOException {
		return null;
	}

    /**
     * Returns the datasource given its datasource resource. This method is an
     * overload of {@link #asDataSource(Resource, Resource)} with context is
     * <code>null</code>.
     * @param datasource the resource representing the datasource
     * @return the datasource given its datasource resource
     * @throws ServletException in case there's a servlet error while fetching data
     * @throws IOException in case there's an i/o error while fetching data
     */
    public DataSource asDataSource(Resource datasource) throws ServletException, IOException {
		return null;
	}

    /**
     * Returns the datasource given its datasource resource.
     *
     * @param datasource The resource representing the datasource
     * @param context The context resource that is returned when calling
     *            {@link SlingHttpServletRequest#getResource()} at the
     *            datasource implementation. If this is <code>null</code>, the
     *            given datasource is used.
     * @return <code>null</code> if the given datasource is <code>null</code>.
     * @throws ServletException in case there's a servlet error while fetching data
     * @throws IOException in case there's an i/o error while fetching data
     */
    public DataSource asDataSource(Resource datasource, Resource context) throws ServletException, IOException {
		return null;
	}

    /**
     * Returns the render condition of the current resource. This method is an
     * overload of {@link #getRenderCondition(Resource)} using the current
     * resource.
     *
     * The render condition is specified by <code>granite:rendercondition</code>
     * or <code>rendercondition</code> subresource.
     *
     * Contrast this with {@link #getRenderCondition(Resource, boolean)}, where
     * only <code>granite:rendercondition</code> is checked. This method is
     * meant for backward compatibility; otherwise it is better to use
     * {@link #getRenderCondition(Resource, boolean)} for performance. Once the
     * transition is over, this method will have the same behaviour as
     * {@link #getRenderCondition(Resource, boolean)} with <code>cache</code> =
     * <code>false</code>.
     *
     * @return the render condition of the current resource
     * @throws ServletException
     *             in case there's a servlet error
     * @throws IOException
     *             in case there's an i/o error
     */
    public RenderCondition getRenderCondition() throws ServletException, IOException {
		return null;
	}

    /**
     * Returns the render condition of the given resource.
     *
     * The render condition is specified by <code>granite:rendercondition</code>
     * or <code>rendercondition</code> subresource.
     *
     * Contrast this with {@link #getRenderCondition(Resource, boolean)}, where
     * only <code>granite:rendercondition</code> is checked. This method is
     * meant for backward compatibility; otherwise it is better to use
     * {@link #getRenderCondition(Resource, boolean)} for performance. Once the
     * transition is over, this method will have the same behaviour as
     * {@link #getRenderCondition(Resource, boolean)} with <code>cache</code> =
     * <code>false</code>.
     *
     * @param resource
     *            the resource
     * @return the render condition of the given resource
     * @throws ServletException
     *             in case there's a servlet error
     * @throws IOException
     *             in case there's an i/o error
     */
    public RenderCondition getRenderCondition(Resource resource) throws ServletException, IOException {
		return null;
	}

    /**
     * Returns the render condition of the given resource.
     *
     * The render condition is specified by <code>granite:rendercondition</code>
     * subresource, unlike {@link #getRenderCondition(Resource)}.
     *
     * @param resource
     *            The resource
     * @param cache
     *            <code>true</code> to cache the result; Use it when checking
     *            render condition of other resource (typically the item
     *            resource) so that the render condition is only resolved once.
     *
     * @return The render condition of the given resource; never <code>null</code>.
     *
     * @throws ServletException
     *             in case there's a servlet error
     * @throws IOException
     *             in case there's an i/o error
     */
    public RenderCondition getRenderCondition(Resource resource, boolean cache) throws ServletException, IOException {
		return null;
	}

    /**
     * Returns the icon class(es) for the given icon string from the content property.
     * @param icon the icon string
     * @return the icon class(es) for the given icon string from the content property
     */
    public String getIconClass(String icon) {
		return null;
	}

    private static String toCamel(String s) {
		return null;
	}

    private static String getResourceType(Resource resource) {
		return null;
	}

    private static String getResourceType(Resource resource, String defaultValue) {
		return null;
	}

    /**
     * Includes the given resource and passes the given tag to its renderer.
     * This method performs similarly to &lt;sling:include resource="" /&gt;.
     * @param resource the resource to include
     * @param tag the tag
     * @throws ServletException in case there's a servlet error
     * @throws IOException in case there's an i/o error
     */
    public void include(Resource resource, Tag tag) throws ServletException, IOException {
	}

    /**
     * Includes the given resource and passes the given options to its renderer.
     * This method performs similarly to &lt;sling:include resource="" /&gt;.
     * @param resource the resource to include
     * @param options the options
     * @throws ServletException in case there's a servlet error
     * @throws IOException in case there's an i/o error
     */
    public void include(Resource resource, Options options) throws ServletException, IOException {
	}

    /**
     * Includes the given resource with the given resourceType and passes the
     * given tag to its renderer. This method performs similarly to
     * &lt;sling:include resource="" resourceType="" /&gt;.
     * @param resource the resource to include
     * @param resourceType the resource type
     * @param tag the tag
     * @throws ServletException in case there's a servlet error
     * @throws IOException in case there's an i/o error
     */
    public void include(Resource resource, String resourceType, Tag tag) throws ServletException, IOException {
	}

    /**
     * Includes the given resource with the given resourceType and passes the
     * given options to its renderer. This method performs similarly to
     * &lt;sling:include resource="" resourceType="" /&gt;.
     * @param resource the resource to include
     * @param resourceType the resource type
     * @param options the options
     * @throws ServletException in case there's a servlet error
     * @throws IOException in case there's an i/o error
     */
    public void include(Resource resource, String resourceType, Options options) throws ServletException, IOException {
	}

    /**
     * Includes the given resource with the given resourceType and passes the
     * given options to its renderer. This method performs similarly to
     * &lt;sling:include resource="" replaceSelectors="" resourceType="" /&gt;.
     * @param resource the resource to include
     * @param resourceType the resource type
     * @param selectors the selectors to be included as part of the request.
     * @param options the options
     * @throws ServletException in case there's a servlet error
     * @throws IOException in case there's an i/o error
     */
    public void include(Resource resource, String resourceType, String selectors, Options options) throws ServletException, IOException {
	}

    /**
     * A convenient overload to
     * {@link #includeForLayout(Resource, Resource, Options)}, with
     * layoutResource as <code>null</code>.
     * @param resource the resource to include
     * @param options the options
     * @throws ServletException in case there's a servlet error
     * @throws IOException in case there's an i/o error
     */
    public void includeForLayout(Resource resource, Options options) throws ServletException, IOException {
	}

    /**
     * Includes the given resource to be rendered by the given layoutResource.
     * This method is used by a component to delegate the rendering process to a
     * layout.
     * <p>
     * If layoutResource is not <code>null</code>, the
     * {@link Options#layoutResource(Resource)} is set.
     * <p>
     * This method will attempt to derive the resourceType to be passed to
     * {@link #include(Resource, String, Options)} based the following
     * priorities:
     * <ol>
     * <li>layoutResource is not null, the resourceType is layoutResource's RT</li>
     * <li>layoutResource is null, the resourceType is {@link Config#LAYOUT}
     * child node's RT</li>
     * <li>the resourceType is default layout as a catch-all fallback</li>
     * </ol>
     * @param resource the resource to include
     * @param layoutResource the layout resource to render the given resource with
     * @param options the options
     * @throws ServletException in case there's a servlet error
     * @throws IOException in case there's an i/o error
     */
    public void includeForLayout(Resource resource, Resource layoutResource, Options options) throws ServletException, IOException {
	}

    /**
     * Calls the given script and passes the given options to its renderer. This
     * method performs similarly to &lt;sling:call script="" /&gt;.
     * @param script the script to be called
     * @param options the options
     * @throws ServletException in case there's a servlet error
     * @throws IOException in case there's an i/o error
     */
    public void call(String script, Options options) throws ServletException, IOException {
	}

    /**
     * An options to be passed to the included resource's renderer.
     */
    public static class Options extends com.adobe.granite.ui.components.Options {
        private Resource layout;

        /**
         * Creates a new instance.
         */
        public Options() {
        }

        /**
         * Sets the tag.
         * @param tag the tag
         * @return the options
         */
        @Override
        public Options tag(Tag tag) {
			return null;
		}

        /**
         * Sets <code>true</code> to make the renderer (the field) should render
         * itself as root field; <code>false</code> otherwise.
         * <p>
         * A root field is a field that acts in its own context, instead of as
         * part of a composite field. For example, sizing field consists of
         * weight and height fields. So sizing field is a composite field and
         * wants to leverage the existing number field for width and height. In
         * this case when sizing field is including (
         * {@link ComponentHelper#include(Resource, Options)}) the number field,
         * it should set this option as <code>false</code>.
         * <p>
         * The field implementation is free to interpret the exact behaviour of
         * root/non-root field. Most likely scenario, the root field will handle
         * it own sizing state (e.g. inline-block/block state), while non root
         * field will not, where the parent composite field is managing it.
         *
         * @param flag the flag
         * @return this instance
         */
        @Override
        public Options rootField(boolean flag) {
			return null;
		}

        /**
         * Returns the layout resource.
         * @return the layout resource
         */
        public Resource layoutResource() {
			return null;
		}

        /**
         * Sets the layout resource.
         * @param r the layout resource to set
         * @return this instance
         */
        public Options layoutResource(Resource r) {
			return null;
		}
    }
}
