/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2016 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.ui.components;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletRequest;

/**
 * A mechanic to allow a certain customization for {@link ExpressionResolver}.
 *
 * <p>
 * The customization is only scoped to the current request, not a global one. The resource type implementation needs to
 * use this class explicitly to make the customization available in the request scope going forward.
 * </p>
 *
 * <p>
 * For example, the resource type implementation of CQ Dialog (<code>/apps/my/dialog.jsp</code>) wants to register
 * <code>cqDesign</code> variable so that it can be used in EL expression:
 * </p>
 *
 * <pre>
 * <code>
 * Design design = getDesign();
 * ExpressionCustomizer customizer = ExpressionCustomizer.from(request);
 * customizer.setVariable("cqDesign", design);
 * </code>
 * </pre>
 *
 * <p>
 * Then in the content structure, the content author can use <code>cqDesign</code> variable in the property that accepts
 * EL:
 * </p>
 *
 * <pre>
 * <code>
 * + /apps/content/my/dialog
 *   - sling:resourceType = "my/dialog"
 *   + field1
 *     - sling:resourceType = "my/textfield"
 *     - value = "${cqDesign.id}"
 * </code>
 * </pre>
 */
public class ExpressionCustomizer {
    private static final String KEY_CACHE = ExpressionCustomizer.class.getName();

    /**
     * Returns the instance from the given request.
     *
     * @param request
     *            the current request acting as the scope of the customization
     * @return the instance of the customizer
     */
    public static ExpressionCustomizer from(ServletRequest request) {
		return null;
	}

    private Map<String, Object> variables = new HashMap<String, Object>();

    /**
     * Returns the value of the variable with the given name.
     *
     * @param name
     *            the name of the variable
     * @return the value of the variable, or <code>null</code> if there is no variable with that name
     */
    public Object getVariable(String name) {
		return null;
	}

    /**
     * Creates a variable with the given name and value.
     *
     * <p>
     * Setting an existing variable with the same name will overwrite its value.
     * </p>
     *
     * @param name
     *            the name of the variable
     * @param value
     *            the value of the variable
     */
    public void setVariable(String name, Object value) {
	}

    /**
     * Returns <code>true</code> if there is a variable with the given name; <code>false</code> otherwise.
     *
     * @param name
     *            the name of the variable
     * @return <code>true</code> if there is a variable with the given name; <code>false</code> otherwise
     */
    public boolean hasVariable(String name) {
		return false;
	}
}
