// =================================================================================================
// ADOBE SYSTEMS INCORPORATED
// Copyright 2012 Adobe Systems Incorporated
// All Rights Reserved
//
// NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the terms
// of the Adobe license agreement accompanying it.
// =================================================================================================
package com.adobe.xmp.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.adobe.xmp.core.impl.XMPConst;
import com.adobe.xmp.core.namespace.XML;

/**
 * Language alternatives allow the text value of a property to be chosen based on a desired
 * language. 
 * XMPLanguageAlternative is implemented as a {@link XMPArray} (Alternate) whose children are
 * {@link XMPSimple} with text value, each of which must have a language qualifier (xml:lang) 
 * associated with it.
 */
public class XMPLanguageAlternative
{

	XMPArray array = null;
	
	/**
	 * Gets underlying {@link XMPArray}
	 * @return XMPArray
	 */
	public XMPArray getArray()
	{
		return null;
	}
	
	/**
	 * Constructs {@link XMPLanguageAlternative} object from the given XMPArray.
	 * Callers of this method must ensure that the XMPArray object sent to this
	 * method should represent a Lang ALT as specified by XMP Specification.
	 * @param array
	 */
	private XMPLanguageAlternative( XMPArray array )
	{
	    this.array = array;
	}
	
	/**
	 * Constructs a {@link XMPLanguageAlternative} which wraps the given {@link XMPArray}. If 
	 * this {@link XMPArray} does not represent a Lang Alt as per XMP Specification this
	 * method returns null.
	 * To detect whether a language alternative, these prerequisites 
	 * must be fulfilled:
	 * <ul>
	 * 		<li>the array contains only simple properties</li>
	 * 		<li>at least one property has an xml:lang qualifier.</li>
	 * </ul>
	 * @param array
	 */
	public static XMPLanguageAlternative newInstance( XMPArray array )
	{
		return null;
	}
	
	/**
	 * This will normalize the alternative array to meet the criteria of a XMPLanguage alternative.
	 * This is only important if a XMPArray is constructed outside of this facade and then used within this context.
	 * If you only use this facade, a currupt XMPArray will never be created.
	 * 
	 * Normalization will do the following:
	 * <ul>
	 * 		<li>x-default is moved to top (first one if multiple)</li>
	 * 		<li>Items with empty languages become the default language, when there is no "x-default". (first one if multiple) </li>
	 * 		<li>Any other item qualifiers than xml:lang are deleted.</li>
	 * 		<li>language will be normalized to RFC 3066 form
	 * </ul>
	 * after cleanup, other x-defaults than the first one and other items with no xml:lang will not be deleted at this point
	 * x-defaults will be deleted while setting a new default. Emptys will be preserved but not accessible with this fassade.
	 */
	public void normalize()
	{
	}
	
	private void cleanupLangAltQualifier( XMPQualifiers qualifiers)
	{
	}
	
	/**
	 * Checks if the Language Alternative is empty
	 * @return True if the array is empty False otherwise
	 */
	public boolean isEmpty()
	{
		return false;
	}
	
	/**
	 * returns the number of elements
	 * @return the number of elements
	 */
	public int size()
	{
		return 0;
	}
	
	/**
	 * Delete all elements
	 */
	public void clear()
	{
	}
	
	private static String getLanguage ( XMPSimple simple )
	{
		return null;
	}
	
	/**
	 * Set or overwrites a localized text entry for this language alternative.
	 * @param language the language String, has to follow the RFC 3066 format
	 * @param value the value for this language alternative entry
	 * @return the localized text entry just created or null if nothing was created
	 * @throws IllegalArgumentException <code>IllegalArgumentException</code> is thrown if value is null
	 */
	public XMPSimple setLocalizedText( String language, String value )
	{
		return null;
	}
	
	/**
	 * remove duplicated x-default entries on positions other that the first one
	 * @param array
	 * @return
	 */
	private XMPArray removeDuplicatedXDefault( XMPArray array )
	{
		return null;
	}
	/**
	 * Set or overwrites the default localized text entry for this language alternative.
	 * @param  language the language of the default entry (or null or x-default. if no specific language is given)
	 * @param value the value for this language alternative entry
	 * @return the localized text entry just created or null if nothing was created
	 */
	public XMPSimple setDefaultText( String language, String value )
	{
		return null;
	}
	
	/**
	 * Sets the default text entry (first one in array) in an language alternative array.
	 * If a new entry must be created (array is empty), x-default will be used as language
	 * @param value the value for this language alternative entry
	 * @return the localized text entry just created or null if nothing was created
	 */
	public XMPSimple setDefaultText(  String value )
	{
		return null;
	}
	
	/**
	 * Sets the entry with a certain language as the default entry (pushes it to the first location)
	 * @param language the language to set as default
	 * @return the new default entry or null if nothing could be set (language entry does not exist)
	 */
	public XMPSimple setDefaultLanguage( String language )
	{
		return null;
	}
	/**
	 * Returns the language alternative array item for a specific language
	 * If language is x-default the first entry of the array is returned, even
	 * if someone accidently added an "x-default" entry
	 * at another position than the first one (not possible while using this class) 
	 * @param language the language for the desired array item
	 * @return the array item requested or null if it does not exist
	 */
	public XMPSimple getLocalizedText( String language )
	{
		return null;
	}
	
	
	/**
	 * Returns the default language from an alternative array.
	 * This is always the first entry (even if someone accidently added an "x-default" entry
	 * at another position than the first one (not possible while using this class)
	 * @return the array item requested or null if it does not exist
	 */
	public XMPSimple getDefaultText( )
	{
		return null;
	}
	
	
	/**
	 * @return Returns a list of languages that are set in this language alternative.
	 */
	public List<String> getAvailableLanguages()
	{
		return null;
	}
	
	
	/**
	 * Removes the specified language item from an alternative array
	 * @param language the language that shall be removed
	 * @return the removed  item or null if it did not exist
	 */
	public XMPSimple removeLocalizedText( String language )
	{
		return null;
	}
	
	/**
	 * removes the default entry (the first one) and the second one will be default
	 * @return the removed item or null if it did not exist
	 */
	public XMPSimple removeDefaultText()
	{
		return null;
	}
	
	/**
	 * Normalize an xml:lang value so that comparisons are effectively case
	 * insensitive as required by RFC 3066 (which superceeds RFC 1766). The
	 * normalization rules:
	 * <ul>
	 * <li> The primary subtag is lower case, the suggested practice of ISO 639.
	 * <li> All 2 letter secondary subtags are upper case, the suggested
	 * practice of ISO 3166.
	 * <li> All other subtags are lower case.
	 * </ul>
	 *
	 * @param value
	 *            raw value
	 * @return Returns the normalized value.
	 */
	private String normalizeLangValue( String value )
	{
		return null;
	}

	public Iterator<XMPNode> iterator() 
	{
		return null;
	}
}
