/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.foundation;

import com.day.cq.commons.Filter;
import com.day.cq.commons.RangeIterator;
import com.day.cq.search.Predicate;
import com.day.cq.search.Query;
import com.day.cq.search.QueryBuilder;
import com.day.cq.search.SimpleSearch;
import com.day.cq.search.result.Hit;
import com.day.cq.search.result.SearchResult;
import com.day.cq.tagging.TagManager;
import com.day.cq.wcm.api.NameConstants;
import com.day.cq.wcm.api.Page;
import com.day.cq.wcm.api.PageFilter;
import com.day.cq.wcm.api.PageManager;
import com.day.text.Text;
import org.apache.commons.collections.iterators.EmptyIterator;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.ValueFormatException;
import javax.jcr.query.QueryResult;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.NoSuchElementException;
import java.util.Set;

/**
 * Creates a list from a resource for use in the list component.
 */
public class List {

    public static final Logger log = LoggerFactory.getLogger(List.class);

    public static final String DEFAULT_QUERY = "";

    public static final String URL_EXTENSION = ".html";

    public static final String TYPE_PROPERTY_NAME = "displayAs";

    public static final String SOURCE_PROPERTY_NAME = "listFrom";

    /**
     * Used to create a list from child pages.
     */
    public static final String SOURCE_CHILDREN = "children";

    /**
     * Used to create a list from descendant pages.
     */
    public static final String SOURCE_DESCENDANTS = "descendants";

    /**
     * Used to create a list from a fixed selection of pages.
     */
    public static final String SOURCE_STATIC = "static";

    /**
     * Used to create a list from a search result.
     */
    public static final String SOURCE_SEARCH = "search";

    /**
     * Used to create a list from a querybuilder search result. Allows rendering of any node, not just pages.
     */
    public static final String SOURCE_QUERYBUILDER = "querybuilder";

    /**
     * Used to create a list from list of tags.
     */
    public static final String SOURCE_TAGS = "tags";

    /**
     * Used for default rendering of the list items.
     */
    public static final String TYPE_DEFAULT = "default";

    public static final String PARENT_PAGE_PROPERTY_NAME = "parentPage";

    public static final String ANCESTOR_PAGE_PROPERTY_NAME = "ancestorPage";

    public static final String TAGS_SEARCH_ROOT_PROPERTY_NAME = "tagsSearchRoot";

    public static final String TAGS_PROPERTY_NAME = "tags";
    
    public static final String TAGS_MATCH_PROPERTY_NAME = "tagsMatch";

    public static final String PAGES_PROPERTY_NAME = "pages";

    public static final String QUERY_PROPERTY_NAME = "query";

    public static final String SEARCH_IN_PROPERTY_NAME = "searchIn";

    public static final String LIMIT_PROPERTY_NAME = "limit";

    public static final String PAGE_MAX_PROPERTY_NAME = "pageMax";

    public static final String ORDER_BY_PROPERTY_NAME = "orderBy";

    public static final String ORDERED_PROPERTY_NAME = "ordered";

    public static final String SAVEDQUERY_PROPERTY_NAME = "savedquery";

    public static final String PAGE_START_PARAM_NAME = "start";

    public static final String PAGE_MAX_PARAM_NAME = "max";

    public static final String ALLOW_DUPLICATES_PROPERTY_NAME = "allowDuplicates";

    private String id;

    private SlingHttpServletRequest request;

    private java.util.List<Resource> resources;

    private java.util.List<Page> pages;

    private Iterator<Page> pageIterator;

    private Iterator<Node> nodeIterator;

    private boolean inited;

    private boolean ordered;

    private String source;

    private String query;

    private String queryType;

    private String startIn;

    private String type;

    private String orderBy;

    private Comparator<Page> orderComparator;

    private int limit = -1;

    private int pageMaximum = -1;

    private int pageStart = 0;

    private Resource resource;

    private ValueMap properties;

    private PageFilter pageFilter;

    /**
     * Creates a <code>List</code> instance based on the specified request.
     * @param request The request
     */
    public List(SlingHttpServletRequest request) {
        this(request, null);
    }

    /**
     * Creates a <code>List</code> instance based on the specified request.
     *
     * @param request The request
     * @param pageFilter Page filter
     */
    public List(SlingHttpServletRequest request, PageFilter pageFilter) {
        this.request = request;
        this.pageFilter = pageFilter;
        resource = request.getResource();
        generateId();
    }

    private void initConfig() {
	}

    @SuppressWarnings("unchecked")
    private boolean init() {
		return false;
	}

    private void generateId() {
	}

    private SimpleSearch getSearch(String path) {
		return null;
	}

    private String getParameter(String name) {
		return null;
	}

    /**
     * Returns the list items as resources.
     * @return The resources
     * @deprecated use {@link List#getPages()} instead
     */
    public Iterator<Resource> getResources() {
		return null;
	}

    /**
     * Returns the list items as pages, respecting both starting index and
     * maximum number of list items if specified.
     * @return The pages
     */
    public Iterator<Page> getPages() {
		return null;
	}

    /**
     * Returns the list as nodes. This is only the case when an advanced search
     * is run, ie. the {@link #setSource(String) source} is
     * {@link #SOURCE_QUERYBUILDER}. Otherwise <code>null</code> will be
     * returned.
     *
     * @return the nodes of a querybuilder search result or <code>null</code>
     * @since 5.3
     */
    public Iterator<Node> getNodes() {
		return null;
	}

    /**
     * States whether the list is ordered.
     * @return <code>true</code> if list is ordered, <code>false</code> otherwise
     */
    public boolean isOrdered() {
		return false;
	}

    /**
     * States whether the list is empty.
     * @return <code>true</code> if list is empty, <code>false</code> otherwise
     */
    public boolean isEmpty() {
		return false;
	}

    /**
     * Returns the number of list items.
     * @return The size of the list
     */
    public int size() {
		return 0;
	}

    /**
     * States whether the list is paginating, i.e. has a starting index
     * and/or a maximum numbe rof list items per page defined.
     * @return <code>true</code> if list is paginating, <code>false</code> otherwise
     */
    public boolean isPaginating() {
		return false;
	}

    /**
     * Returns the starting index for list items on this page.
     * Used for pagination.
     * @return The start index of
     */
    public int getPageStart() {
		return 0;
	}

    /**
     * Returns the maximum of list items to return per page.
     * Used for pagination.
     * @return The maximum per page
     */
    public int getPageMaximum() {
		return 0;
	}

    /**
     * Returns the link to the page with the next set of list items.
     * Used for pagination.
     * @return The link to the next page
     */
    public String getNextPageLink() {
		return null;
	}

    /**
     * Returns the link to the page with the previous set of list items.
     * Used for pagination.
     * @return The link to the previous page
     */
    public String getPreviousPageLink() {
		return null;
	}

    /**
     * Sets the source for the list. The value can be one of:
     * <ul>
     * <li>{@link #SOURCE_CHILDREN}</li>
     * <li>{@link #SOURCE_SEARCH}</li>
     * <li>{@link #SOURCE_STATIC}</li>
     * <li>{@link #SOURCE_QUERYBUILDER}</li>
     * </ul>
     * This is an optional setter, the default value will be
     * taken from the property defined by {@link #SOURCE_PROPERTY_NAME}.
     * @param src The list source
     */
    public void setSource(String src) {
	}

    /**
     * Sets the query for the fulltext search.
     * @param q The query
     */
    public void setQuery(String q) {
	}

    /**
     * Sets the query for the search with the specified type (xpath, sql).
     * @param q The query
     * @param type The query type (optional)
     */
    public void setQuery(String q, String type) {
	}

    /**
     * Sets the the of the page to start searching on.
     * @param start The start page path
     */
    public void setStartIn(String start) {
	}

    /**
     * Returns the type of the list. This can be used to select the
     * script that renders the list items.
     * @return The list type
     */
    public String getType() {
		return null;
	}

    /**
     * Sets the type of the list.
     * This is an optional setter, the default value will be
     * taken from the property defined by {@link #TYPE_PROPERTY_NAME}.
     * @param t The list type
     */
    public void setType(String t) {
	}

    /**
     * Sets the comparator used to order the list with.
     * This is an optional setter and overrides #setOrderBy(String).
     * @param obc The comparator to order the list with
     */
    public void setOrderComparator(Comparator<Page> obc) {
	}

    /**
     * Sets the property to order the list by.
     * This is an optional setter, the default value will be
     * taken from the property defined by {@link #ORDER_BY_PROPERTY_NAME}.
     * @param ob The property to order the list by
     */
    public void setOrderBy(String ob) {
	}

    /**
     * Sets the list style (numbered or not).
     * This is an optional setter, the default value will be <code>false</code>
     * @param o <code>true</code> if list should be rendered with numbered items,
     *          <code>false</code> otherwise
     */
    public void setOrdered(boolean o) {
	}

    /**
     * Sets the limit for list items.
     * This is an optional setter, the default value will be
     * taken from the property defined by {@link #LIMIT_PROPERTY_NAME}.
     * @param l The limit
     */
    public void setLimit(int l) {
	}

    /**
     * Sets the maximum list items to return. Enables pagination.
     * This is an optional setter, the default value will be
     * taken from the property defined by {@link #PAGE_MAX_PROPERTY_NAME}.
     * @param pm The maximum number of pages per page
     */
    public void setPageMaximum(int pm) {
	}

    /**
     * Sets the starting index to be respected when returning
     * list items. Used when paginating.
     * @param psi The starting index (defaults to 0).
     */
    public void setPageStart(int psi) {
	}


    /**
     * Sets the page iterator to generate the list from.
     * @param iter The iterator
     */
    public void setPageIterator(Iterator<Page> iter) {
	}

    /**
     * Compares pages with each other by property.
     */
    public class PageComparator<P extends Page> implements Comparator<P> {

        private String property;

        /**
         * Creates a <code>PageComparator</code> instance using the
         * specified property for comparison.
         * @param prop The name of the property
         */
        public PageComparator(String prop) {
            property = prop;
        }

        /**
         * {@inheritDoc}
         */
        public int compare(P p1, P p2) {
			return 0;
		}

        private String getKey(P p) {
			return null;
		}

        /*
         * Checks if the property 'prop' of the given page is a JCR property of type Date.
         */
        private boolean isDate(Page page, String prop) {
			return false;
		}

    }

    /**
     * A helper class to create links with query parameters.
     */
    private class PageLink {

        private String path;
        private HashMap<String, Object> params;

        public PageLink(SlingHttpServletRequest request) {
            path = request.getPathInfo();

            // Get containing page from path and replace path with its path
            PageManager pm = request.getResourceResolver().adaptTo(PageManager.class);
            Page page = pm.getContainingPage(path);
            if (page != null) {
                path = page.getPath() + URL_EXTENSION;
            }

            initParams(request.getQueryString());
        }

        public void addParameter(String name, Object value) {
		}

        public void setParameter(String name, Object value) {
		}

        public String toString() {
			return null;
		}

        private String prefixName(String name) {
			return null;
		}

        private void initParams(String query) {
		}

        private String appendParam(String url, String name, Object value) {
			return null;
		}

    }

    /**
     * An abstract base class for page iterators.
     */
    private abstract class AbstractPageIterator implements Iterator<Page> {

        protected PageManager pm;
        protected Page nextPage;
        // keeps track of pages seen
        protected Set<String> seen;
        // Filter pages
        protected PageFilter pageFilter;

        public boolean hasNext() {
			return false;
		}

        public Page next() {
			return null;
		}

        protected abstract Page seek() {
			return null;
		}

        public void remove() {
		}

    }


    /**
     * A helper class that iterates over the pages behind a node iterator.
     */
    private class NodeBasedPageIterator extends AbstractPageIterator {

        private NodeIterator nodes;

        public NodeBasedPageIterator(PageManager pm, NodeIterator nodes, boolean avoidDuplicates, PageFilter pageFilter) {
            this.pm = pm;
            this.nodes = nodes;
            if (avoidDuplicates) {
                this.seen = new HashSet<String>();
            }
            this.pageFilter = pageFilter;
            this.nextPage = seek();
        }

        protected Page seek() {
			return null;
		}

    }

    /**
     * A helper class that iterates over the pages behind a hit iterator.
     */
    private class HitBasedPageIterator extends AbstractPageIterator {

        private Iterator<Hit> hits;

        public HitBasedPageIterator(PageManager pm, Iterator<Hit> hits, boolean avoidDuplicates, PageFilter pageFilter) {
            this.pm = pm;
            this.hits = hits;
            if (avoidDuplicates) {
                this.seen = new HashSet<String>();
            }
            this.pageFilter = pageFilter;
            this.nextPage = seek();
        }

        protected Page seek() {
			return null;
		}

    }

}
