/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.image;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;

/**
 * The ResizeOp class implements a weighted resize filter which produces better
 * results than the scaling AffineTransformOp and is faster than a blurring
 * ConvolveOp with a following scaling AffineTransformOp.
 * <p>
 * The <code>RenderingHints</code> defined at construction time are used if the
 * destination color model has to be adapted for the filter operation.
 * <p>
 * Note that the following constraints have to be met
 * <ul>
 * <li>The <code>source</code> and <code>destination</code> must be different.
 * </ul>
 *
 * @see <a href="doc-files/ResizeImplementation.html">Implementation of Image
 *      Resizing</a>
 * @version $Revision$
 * @author tripod
 * @author fmeschbe
 * @since coati
 * @audience wad
 */
public class ResizeOp extends AbstractBufferedImageOp {

    /** The horizontal scale factor */
    private final double scaleX;

    /** The vertical scale factor */
    private final double scaleY;

    /**
     * use faster algorithm if specified
     */
    private boolean fast;

    /**
     * Creates a new <code>ResizeOp</code> object.
     *
     * @param scaleX The horizontal scale factor
     * @param scaleY The vertical scale factor
     * @param hints The rendering hints. May be <code>null</code>.
     */
    public ResizeOp(double scaleX, double scaleY, RenderingHints hints) {
        super(hints);
        this.scaleX = scaleX;
        this.scaleY = scaleY;
    }

    /**
     * Creates a new <code>ResizeOp</code> with no rendering hints.
     *
     * @param scaleX The horizontal scale factor
     * @param scaleY The vertical scale factor
     */
    public ResizeOp(double scaleX, double scaleY) {
        this(scaleX, scaleY, null);
    }

    /**
     * Creates a new <code>ResizeOp</code> with no rendering hints and the same
     * horizontal and vertical scale factor.
     *
     * @param scale The scale factor used for both horizontal and vertical
     *            scaling.
     */
    public ResizeOp(double scale) {
        this(scale, scale);
    }

    // ---------- BufferedImageOp interface
    // -------------------------------------

    /**
     * Returns the bounding box of the filtered destination image. The
     * IllegalArgumentException may be thrown if the source image is
     * incompatible with the types of images allowed by the class implementing
     * this filter.
     */
    public Rectangle2D getBounds2D(BufferedImage src) {
		return null;
	}

    /**
     * Returns the location of the destination point given a point in the source
     * image. If dstPt is non-null, it will be used to hold the return value.
     */
    public Point2D getPoint2D(Point2D srcPt, Point2D dstPt) {
		return null;
	}

    public boolean isFast() {
		return false;
	}

    public void setFast(boolean fast) {
	}

    // ---------- protected
    // -----------------------------------------------------

    /**
     * Implements the resize operation.
     *
     * @param src The source image to be operated upon.
     * @param dst The destination image getting the resulting image. This must
     */
    protected void doFilter(BufferedImage src, BufferedImage dst) {
	}

    /**
     * Implements the resizing using Java2D with bicubic interpolation. note
     * that this is only supported in some jdk 1.5 JVMs
     *
     * @param src The source image to be operated upon.
     * @param dst The destination image getting the resulting image. This must
     */
    private static void doFilter_bicubic(BufferedImage src, BufferedImage dst) {
	}

    /**
     * Implements the resizing using Java2D with bilinear interpolation.
     *
     * @param src The source image to be operated upon.
     * @param dst The destination image getting the resulting image. This must
     */
    private static void doFilter_bilinear(BufferedImage src, BufferedImage dst) {
	}

    /**
     * Implements the resizing using Java2D with a progressive bilinear
     * algorithm.
     *
     * @param src The source image to be operated upon.
     * @param dst The destination image getting the resulting image. This must
     */
    static void doFilter_progressive(BufferedImage src,
            BufferedImage dst) {
			}

    /**
     * Implements the weighted filter algorithm. This method does not depend on
     * the source and destination to have the same color model, as it operates
     * on the RGB color value extracted with the <code>getRGB()</code> method
     * and sets the destination with <code>setRGB()</code> which internally
     * convert to the real color model.
     * <p>
     * The source and destination must not be the same and not <code>null</code>.
     *
     * @param src The source image which is to be scaled.
     * @param dst The destination image getting the scaled image. This must not
     *            be <code>null</code>.
     */
    private static void doFilter_weighted(BufferedImage src, BufferedImage dst) {
	}

    /**
     * Divides 2 integers and rounds up the result to the next integer.
     *
     * Examples:
     * - a/b = 2.32: the result is 3
     * - a/b = 2.00: the result is 2
     *
     * @param a The dividend
     * @param b The divisor
     * @return The result of a/b rounded up to the next integer
     */
    private static int divideAndRoundUp(int a, int b) {
		return 0;
	}

}
