/*
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2017 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 */
package com.day.util.diff;

import java.io.IOException;

/**
 * A hunk records a block of diff between the left and the right document.
 * it represents either a insertion, deletion, change or identity. several
 * hunks are chained in a linked list.
 */
public class Hunk {

    /**
     * type indicates an unmodified block
     */
    public static final int UNMODIFIED = 0;

    /**
     * type indicates an inserted block
     */
    public static final int INSERTED = 1;

    /**
     * type indicates a deleted block
     */
    public static final int DELETED = 2;

    /**
     * type indicates a changed block
     */
    public static final int CHANGED = INSERTED | DELETED;

    /**
     * the range in the left document
     */
    public final Range left;

    /**
     * the rnage in the right document
     */
    public final Range right;

    /**
     * the hunk type
     */
    public final int type;

    /**
     * the previous hunk
     */
    private Hunk prev;

    /**
     * the next hunk
     */
    private Hunk next;

    /**
     * Creates a new hunk and adds it to the previous hunk
     * @param left the left range
     * @param right the right range
     * @param type the hunk type
     * @param prev the previous hunk
     */
    public Hunk(Range left, Range right, int type, Hunk prev) {
        this.left = left;
        this.right = right;
        this.prev = prev;
        this.type = type;
        if (prev != null) {
            prev.next = this;
        }
    }

    /**
     * Returns the previous hunk of this chain
     * @return the previous hunk.
     */
    public Hunk prev() {
		return null;
	}

    /**
     * Returns the next hunk of this chain
     * @return the next hunk.
     */
    public Hunk next() {
		return null;
	}

    /**
     * Writes a unified diff to the given writer and returns the next hunk to
     * continue the output.
     *
     * @param out the writer
     * @param numContextLines the number of context lines to include in the
     *        diff blocks. do not change during iteration!
     * @return the next hunk or <code>null</code>.
     * @throws IOException if an I/O error occurs.
     */
    public Hunk write(DiffWriter out, int numContextLines) throws IOException {
		return null;
	}

    /**
     * Writes the body of this hunk
     * @param out the writer
     * @throws IOException if an I/O error occurs.
     */
    private void writeBody(DiffWriter out) throws IOException {
	}

    /**
     * Write the actual lines
     * @param out the writer
     * @param range the range
     * @param prefix the prefix
     * @throws IOException if an I/O error occurs
     */
    private static void writeData(DiffWriter out, Range range, String prefix)
            throws IOException {
			}
}