/*
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2017 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 */
package com.day.util.diff;

import java.io.IOException;

/**
 * A hunk3 represents a block of a change from the 3-way diff. it can either be
 * a modification on the left side (for which {@link #getLeftRange()} is not
 * <code>null</code>), or a change on the right side (for which
 * {@link #getRightRange()} is not <code>null</code>. if the left and the right
 * ranges are <code>null</code> this hunk represents an unmodified block of the
 * base document. if both ranges are NOT <code>null</code> it represents a
 * conflicting block.
 */
public class Hunk3 {

    /**
     * the left document marker pattern
     */
    public static final String[] MARKER_L  = new String[]{"<<<<<<< ", ".mine"};

    /**
     * the right document marker pattern
     */
    public static final String[] MARKER_R  = new String[]{">>>>>>> ", ".theirs"};

    /**
     * the base document marker pattern
     */
    public static final String[] MARKER_B  = new String[]{"||||||| ", ".base"};

    /**
     * the separation marker pattern
     */
    public static final String[] MARKER_M  = new String[]{"=======", ""};

    /**
     * the base range. never <code>null</code>.
     */
    private final Range base;

    /**
     * the left range
     */
    private final Range left;

    /**
     * the right range
     */
    private final Range right;

    /**
     * the next hunk in this chain
     */
    private Hunk3 next;

    /**
     * Constructs a new hunk and appends it to the previous one.
     * @param base the base document
     * @param left the left document
     * @param right the right document
     * @param prev the previous hunk
     */
    public Hunk3(Range base, Range left, Range right, Hunk3 prev) {
        this.base = base;
        this.left = left;
        this.right = right;
        if (prev != null) {
            prev.next = this;
        }
    }

    /**
     * Returns the next hunk in this chain or <code>null</code> of this hunk is
     * the last one.
     * @return the next hunk.
     */
    public Hunk3 next() {
		return null;
	}

    /**
     * Returns the range of the base document this hunk spans.
     * @return the base range.
     */
    public Range getBaseRange() {
		return null;
	}

    /**
     * Returns the range of the left document this hunk spans.
     * can be <code>null</code>.
     * @return the left range.
     */
    public Range getLeftRange() {
		return null;
	}

    /**
     * Returns the range of the right document this hunk spans.
     * can be <code>null</code>.
     * @return the right range.
     */
    public Range getRightRange() {
		return null;
	}

    /**
     * Writes this hunk to a writer. if this hunk represents a conflict it is
     * included in a merge-like manner using the separators defined above.
     *
     * @param out the writer
     * @param showBase include the block of the base document in case of a
     *        conflict as well.
     * @throws IOException if a I/O error occurs.
     */
    public void write(DiffWriter out, boolean showBase) throws IOException {
	}

    /**
     * generate a debug string
     * @return a debug string
     */
    public String toString() {
		return null;
	}

    /**
     * adds some line numbers to the string buffer
     * @param buf the buffer
     * @param i the index
     */
    private void addLineNumbers(StringBuffer buf, int i) {
	}

    /**
     * Returns the marker string for the given format and document
     * @param fmt the marker format
     * @param doc the document or <code>null</code>.
     * @return the marker string
     */
    public static String getMarker(String[] fmt, Document doc) {
		return null;
	}

}