/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.jackrabbit.oak.cache;

import static org.apache.jackrabbit.oak.commons.IOUtils.humanReadableByteCount;

import java.util.Map;
import java.util.concurrent.TimeUnit;

import com.google.common.base.Objects;
import com.google.common.cache.Cache;
import com.google.common.cache.Weigher;
import org.apache.jackrabbit.oak.api.jmx.CacheStatsMBean;
import org.apache.jackrabbit.oak.commons.jmx.AnnotatedStandardMBean;

/**
 * Cache statistics.
 */
public class CacheStats extends AnnotatedStandardMBean implements CacheStatsMBean {
    private final Cache<Object, Object> cache;
    private final Weigher<Object, Object> weigher;
    private final long maxWeight;
    private final String name;
    private com.google.common.cache.CacheStats lastSnapshot = 
            new com.google.common.cache.CacheStats(
            0, 0, 0, 0, 0, 0);

    /**
     * Construct the cache stats object.
     * 
     * @param cache the cache
     * @param name the name of the cache
     * @param weigher the weigher used to estimate the current weight
     * @param maxWeight the maximum weight
     */
    @SuppressWarnings("unchecked")
    public CacheStats(Cache<?, ?> cache, String name, 
            Weigher<?, ?> weigher, long maxWeight) {
        super(CacheStatsMBean.class);
        this.cache = (Cache<Object, Object>) cache;
        this.name = name;
        this.weigher = (Weigher<Object, Object>) weigher;
        this.maxWeight = maxWeight;
    }

    @Override
    public long getRequestCount() {
		return 0L;
	}

    @Override
    public long getHitCount() {
		return 0L;
	}

    @Override
    public double getHitRate() {
		return 0.0d;
	}

    @Override
    public long getMissCount() {
		return 0L;
	}

    @Override
    public double getMissRate() {
		return 0.0d;
	}

    @Override
    public long getLoadCount() {
		return 0L;
	}

    @Override
    public long getLoadSuccessCount() {
		return 0L;
	}

    @Override
    public long getLoadExceptionCount() {
		return 0L;
	}

    @Override
    public double getLoadExceptionRate() {
		return 0.0d;
	}

    @Override
    public long getTotalLoadTime() {
		return 0L;
	}

    @Override
    public double getAverageLoadPenalty() {
		return 0.0d;
	}

    @Override
    public long getEvictionCount() {
		return 0L;
	}

    @Override
    public long getElementCount() {
		return 0L;
	}

    @Override
    public long estimateCurrentWeight() {
		return 0L;
	}

    @Override
    public long getMaxTotalWeight() {
		return 0L;
	}

    @Override
    public synchronized void resetStats() {
	}

    @Override
    public String cacheInfoAsString() {
		return null;
	}

    @Override
    public String getName() {
		return null;
	}

    private com.google.common.cache.CacheStats stats() {
		return null;
	}

    static String timeInWords(long nanos) {
		return null;
	}
}
