/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.commons.io;

import java.io.FilterInputStream;
import java.io.IOException;

import com.google.common.io.ByteSource;
import org.apache.commons.io.input.ClosedInputStream;

/**
 * * This input stream delays accessing the ByteSource until the first byte is read
 */
public class LazyInputStream extends FilterInputStream {
    private final ByteSource byteSource;
    private boolean opened;

    public LazyInputStream(ByteSource byteSource) {
        super(null);
        this.byteSource = byteSource;
    }

    @Override
    public int read() throws IOException {
		return 0;
	}

    @Override
    public int read(byte[] b) throws IOException {
		return 0;
	}

    @Override
    public int read(byte[] b, int off, int len) throws IOException {
		return 0;
	}

    @Override
    public long skip(long n) throws IOException {
		return 0L;
	}

    @Override
    public int available() throws IOException {
		return 0;
	}

    @Override
    public void close() throws IOException {
	}

    @Override
    public synchronized void mark(int readlimit) {
	}

    @Override
    public synchronized void reset() throws IOException {
	}

    @Override
    public boolean markSupported() {
		return false;
	}

    private void ensureOpen() throws IOException {
	}

    private void ensureOpenWithUnCheckedException(){
	}
}
