/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.namepath;

import javax.jcr.nodetype.ConstraintViolationException;

import org.apache.jackrabbit.util.XMLChar;

/**
 * Parses and validates JCR names. Upon successful completion of
 * {@link #parse(String, Listener, int)}
 * the specified listener is informed about the (resulting) JCR name.
 * In case of failure {@link JcrNameParser.Listener#error(String)} is called indicating
 * the reason.
 */
public final class JcrNameParser {

    // constants for parser
    private static final int STATE_PREFIX_START = 0;
    private static final int STATE_PREFIX = 1;
    private static final int STATE_NAME_START = 2;
    private static final int STATE_NAME = 3;
    private static final int STATE_URI_START = 4;
    private static final int STATE_URI = 5;

    /**
     * Listener interface for this name parser.
     */
    interface Listener {

        /**
         * Informs this listener that parsing the jcr name failed.
         *
         * @param message Details about the error.
         * @see JcrNameParser#parse(String, Listener, int)
         */
        void error(String message) {
		}

        /**
         * Informs this listener about the result of
         * {@link JcrNameParser#parse(String, Listener, int)}
         *
         * @param name The resulting name upon successful completion of
         * {@link org.apache.jackrabbit.oak.namepath.JcrNameParser#parse(String, Listener, int)}
         * @param index the index (or {@code 0} when not specified)
         */
        boolean name(String name, int index) {
			return false;
		}
    }

    /**
     * Avoid instantiation
     */
    private JcrNameParser() {
    }

    /**
     * Parse the specified jcr name and inform the specified {@code listener}
     * about the result or any error that may occur during parsing.
     *
     * @param jcrName The jcr name to be parsed.
     * @param listener The listener to be informed about success or failure.
     * @param index index, or {@code 0} when not specified
     * @return whether parsing was successful
     */
    public static boolean parse(String jcrName, Listener listener, int index) {
		return false;
	}

    public static boolean validate(String jcrName) {
		return false;
	}

    public static void checkName(String jcrName, boolean allowResidual) throws ConstraintViolationException {
	}
}
