/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.client;

import java.io.InputStream;
import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.Calendar;

import javax.jcr.Binary;
import javax.jcr.Item;
import javax.jcr.ItemVisitor;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.lock.Lock;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;

import org.apache.jackrabbit.rmi.remote.RemoteLock;
import org.apache.jackrabbit.rmi.remote.RemoteNode;
import org.apache.jackrabbit.rmi.remote.RemoteProperty;
import org.apache.jackrabbit.rmi.value.SerialValueFactory;

/**
 * Local adapter for the JCR-RMI
 * {@link org.apache.jackrabbit.rmi.remote.RemoteNode RemoteNode}
 * interface. This class makes a remote node locally available using
 * the JCR {@link javax.jcr.Node Node} interface.
 *
 * @see javax.jcr.Node
 * @see org.apache.jackrabbit.rmi.remote.RemoteNode
 */
public class ClientNode extends ClientItem implements Node {

    /** The adapted remote node. */
    private RemoteNode remote;

    /**
     * Creates a local adapter for the given remote node.
     *
     * @param session current session
     * @param remote  remote node
     * @param factory local adapter factory
     */
    public ClientNode(
            Session session, RemoteNode remote, LocalAdapterFactory factory) {
        super(session, remote, factory);
        this.remote = remote;
    }

    /**
     * Returns <code>true</code> without contacting the remote node.
     *
     * {@inheritDoc}
     */
    public boolean isNode() {
		return false;
	}

    /**
     * Calls the {@link ItemVisitor#visit(Node) ItemVisitor.visit(Node)}
     * method of the given visitor. Does not contact the remote node, but
     * the visitor may invoke other methods that do contact the remote node.
     *
     * {@inheritDoc}
     */
    public void accept(ItemVisitor visitor) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public Node addNode(String path) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Node addNode(String path, String type) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public void orderBefore(String src, String dst) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public Property setProperty(String name, Value value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, Value[] values)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, String[] strings)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, String value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, InputStream value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, boolean value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, double value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, long value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, Calendar value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, Node value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, Binary value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, BigDecimal value)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Node getNode(String path) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public NodeIterator getNodes() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public NodeIterator getNodes(String pattern) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public NodeIterator getNodes(String[] globs) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Property getProperty(String path) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public PropertyIterator getProperties() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public PropertyIterator getProperties(String pattern)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public PropertyIterator getProperties(String[] globs)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Item getPrimaryItem() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public String getIdentifier() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public String getUUID() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public PropertyIterator getReferences() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public PropertyIterator getReferences(String name)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public boolean hasNode(String path) throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean hasProperty(String path) throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean hasNodes() throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean hasProperties() throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public NodeType getPrimaryNodeType() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public NodeType[] getMixinNodeTypes() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public boolean isNodeType(String type) throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public void addMixin(String name) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void removeMixin(String name) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public boolean canAddMixin(String name) throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public NodeDefinition getDefinition() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Version checkin() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public void checkout() throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void update(String workspace) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public NodeIterator merge(String workspace, boolean bestEffort)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public void cancelMerge(Version version) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void doneMerge(Version version) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public String getCorrespondingNodePath(String workspace)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public int getIndex() throws RepositoryException {
		return 0;
	}

    /** {@inheritDoc} */
    public void restore(String version, boolean removeExisting)
            throws RepositoryException {
			}

    /** {@inheritDoc} */
    public void restore(Version version, boolean removeExisting)
            throws RepositoryException {
			}

    /** {@inheritDoc} */
    public void restore(Version version, String path, boolean removeExisting)
            throws RepositoryException {
			}

    /** {@inheritDoc} */
    public void restoreByLabel(String label, boolean removeExisting)
            throws RepositoryException {
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, String[] strings, int type)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, Value[] values, int type)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, Value value, int type)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Property setProperty(String name, String string, int type)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public boolean isCheckedOut() throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public VersionHistory getVersionHistory() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Version getBaseVersion() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Lock lock(boolean isDeep, boolean isSessionScoped)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Lock getLock() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public void unlock() throws RepositoryException {
	}

    /** {@inheritDoc} */
    public boolean holdsLock() throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean isLocked() throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public void followLifecycleTransition(String transition)
            throws RepositoryException {
			}

    /** {@inheritDoc} */
    public String[] getAllowedLifecycleTransistions()
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public NodeIterator getSharedSet() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public PropertyIterator getWeakReferences() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public PropertyIterator getWeakReferences(String name)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public void removeShare() throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void removeSharedSet() throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void setPrimaryType(String nodeTypeName)
            throws RepositoryException {
			}

}
