/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.client;

import java.rmi.RemoteException;

import javax.jcr.Value;
import javax.jcr.nodetype.PropertyDefinition;

import org.apache.jackrabbit.rmi.remote.RemotePropertyDefinition;

/**
 * Local adapter for the JCR-RMI
 * {@link org.apache.jackrabbit.rmi.remote.RemotePropertyDefinition RemotePropertyDefinition}
 * interface. This class makes a remote property definition locally available
 * using the JCR {@link javax.jcr.nodetype.PropertyDefinition PropertyDef} interface.
 *
 * @see javax.jcr.nodetype.PropertyDefinition
 * @see org.apache.jackrabbit.rmi.remote.RemotePropertyDefinition
 */
public class ClientPropertyDefinition extends ClientItemDefinition implements PropertyDefinition {

    /** The adapted remote property. */
    private RemotePropertyDefinition remote;

    /**
     * Creates a local adapter for the given remote property definition.
     *
     * @param remote remote property definition
     * @param factory local adapter factory
     */
    public ClientPropertyDefinition(
            RemotePropertyDefinition remote, LocalAdapterFactory factory) {
        super(remote, factory);
        this.remote = remote;
    }

    /** {@inheritDoc} */
    public int getRequiredType() {
		return 0;
	}

    /** {@inheritDoc} */
    public String[] getValueConstraints() {
		return null;
	}

    /** {@inheritDoc} */
    public Value[] getDefaultValues() {
		return null;
	}

    /** {@inheritDoc} */
    public boolean isMultiple() {
		return false;
	}

    /** {@inheritDoc} */
    public String[] getAvailableQueryOperators() {
		return null;
	}

    /** {@inheritDoc} */
    public boolean isFullTextSearchable() {
		return false;
	}

    /** {@inheritDoc} */
    public boolean isQueryOrderable() {
		return false;
	}

}
