/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.name;

import javax.jcr.RepositoryException;

import org.apache.jackrabbit.spi.Path;

/**
 * Abstract base class for paths.
 */
abstract class AbstractPath implements Path, Path.Element {

    /** Serial version UID */
    private static final long serialVersionUID = 3018771833963770499L;

    /**
     * Returns {@link Path#INDEX_UNDEFINED}, except when overridden by the
     * {@link NamePath} subclass.
     *
     * @return {@link Path#INDEX_UNDEFINED}
     */
    public int getIndex() {
		return 0;
	}

    /**
     * Returns {@link Path#INDEX_DEFAULT}, except when overridden by the
     * {@link NamePath} subclass.
     *
     * @return {@link Path#INDEX_DEFAULT}
     */
    public int getNormalizedIndex() {
		return 0;
	}

    /**
     * Returns <code>null</code>, except when overridden by the
     * {@link IdentifierPath} subclass.
     *
     * @return <code>null</code>
     */
    public String getIdentifier() {
		return null;
	}

    /**
     * Returns <code>false</code>, except when overridden by the
     * {@link RootPath} subclass.
     *
     * @return <code>false</code>
     */
    public boolean denotesRoot() {
		return false;
	}

    /**
     * Returns <code>false</code>, except when overridden by the
     * {@link IdentifierPath} subclass.
     *
     * @return <code>false</code>
     */
    public boolean denotesIdentifier() {
		return false;
	}

    /**
     * Returns <code>false</code>, except when overridden by the
     * {@link ParentPath} subclass.
     *
     * @return <code>false</code>
     */
    public boolean denotesParent() {
		return false;
	}

    /**
     * Returns <code>false</code>, except when overridden by the
     * {@link CurrentPath} subclass.
     *
     * @return <code>false</code>
     */
    public boolean denotesCurrent() {
		return false;
	}

    /**
     * Returns <code>false</code>, except when overridden by the
     * {@link NamePath} subclass.
     *
     * @return <code>false</code>
     */
    public boolean denotesName() {
		return false;
	}

    public Element getNameElement() {
		return null;
	}

    /**
     * Returns this path, except when overridden by the {@link RelativePath}
     * subclasses.
     *
     * @return this path
     */
    public AbstractPath getLastElement() {
		return null;
	}

    /**
     * Returns <code>null</code>, except when overridden by the
     * {@link RelativePath} subclass.
     *
     * @return <code>null</code>
     */
    public Path getFirstElements() {
		return null;
	}

    public final Path resolve(Element element) {
		return null;
	}

    public final Path resolve(Path relative) {
		return null;
	}

    /**
     * Computes the relative path from this path to the given other path.
     * Both paths must be absolute.
     *
     * @param other other path
     * @return relative path
     * @throws RepositoryException if the relative path can not be computed
     */
    public final Path computeRelativePath(Path other)
            throws RepositoryException {
				return null;
			}

    /**
     * Determines if this path is equivalent to the given other path by
     * comparing the normalized paths for equality.
     *
     * @param other other path
     * @return <code>true</code> if this path is equivalent to the other path,
     *         <code>false</code> otherwise
     * @throws IllegalArgumentException if the other path is <code>null</code>
     * @throws RepositoryException if an error occurs
     */
    public final boolean isEquivalentTo(Path other)
            throws IllegalArgumentException, RepositoryException {
				return false;
			}

    /**
     * Determines if this path is a ancestor of the given other path
     * by comparing the depths of the paths and checking if the corresponding
     * ancestor of the given other path is equivalent to this path.
     *
     * @param other other path
     * @return <code>true</code> if this path is an ancestor of the other path,
     *         <code>false</code> otherwise
     * @throws IllegalArgumentException if the other path is <code>null</code>,
     *                                  or relative when this path is absolute,
     *                                  or vice versa
     * @throws RepositoryException if an error occurs
     */
    public final boolean isAncestorOf(Path other)
            throws IllegalArgumentException, RepositoryException {
				return false;
			}

    /**
     * Determines if this path is a descendant of the given other path
     * by comparing the depths of the paths and checking if the corresponding
     * ancestor of this path is equivalent to the given other path.
     *
     * @param other other path
     * @return <code>true</code> if this path is a descendant of the other path,
     *         <code>false</code> otherwise
     * @throws IllegalArgumentException if the other path is <code>null</code>,
     *                                  or relative when this path is absolute,
     *                                  or vice versa
     * @throws RepositoryException if an error occurs
     */
    public final boolean isDescendantOf(Path other)
            throws IllegalArgumentException, RepositoryException {
				return false;
			}

    //--------------------------------------------------------------< Object >

    /**
     * Returns the string representation of this path.
     *
     * @return path string
     */
    public final String toString() {
		return null;
	}

}
