/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.value;

import java.io.InputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Calendar;

import javax.jcr.Binary;
import javax.jcr.Node;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.ValueFormatException;

/**
 * This class implements the <code>ValueFactory</code> interface.
 *
 * @see javax.jcr.Session#getValueFactory()
 */
public abstract class AbstractValueFactory implements ValueFactory {

    /**
     * Constructs a <code>ValueFactory</code> object.
     */
    protected AbstractValueFactory() {
    }

    /**
     * Checks the format of the given string representing a path value.
     * Implementations must assert that the given value is a valid jcr path.
     *
     * @param pathValue
     * @throws javax.jcr.ValueFormatException If the given <code>pathValue</code>
     * isn't a valid jcr path.
     */
    protected abstract void checkPathFormat(String pathValue) throws ValueFormatException {
	}

    /**
     * Checks the format of the given string representing a name value.
     * Implementations must assert that the given value is a valid jcr name.
     *
     * @param nameValue
     * @throws javax.jcr.ValueFormatException  If the given <code>pathValue</code>
     * isn't a valid jcr name.
     */
    protected abstract void checkNameFormat(String nameValue) throws ValueFormatException {
	}

    //---------------------------------------------------------< ValueFactory >
    /**
     * {@inheritDoc}
     */
    public Value createValue(boolean value) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(Calendar value) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(double value) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(InputStream value) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(long value) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(Node value) throws RepositoryException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(String value) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(String value, int type)
            throws ValueFormatException {
				return null;
			}

    /**
     * {@inheritDoc}
     */
    public Binary createBinary(InputStream stream) throws RepositoryException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(Binary value) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(BigDecimal value) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Value createValue(Node node, boolean weak)
            throws RepositoryException {
				return null;
			}

}