/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.vault.fs.io;

import org.apache.jackrabbit.vault.fs.api.PathMapping;
import org.apache.jackrabbit.vault.fs.api.VaultInputSource;
import org.apache.jackrabbit.vault.fs.config.MetaInf;
import org.apache.jackrabbit.vault.util.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Implements an archive that remaps the entries of an underlying archive using a {@link PathMapping}.
 */
public class MappedArchive extends AbstractArchive {

    /**
     * default logger
     */
    private static final Logger log = LoggerFactory.getLogger(MappedArchive.class);

    private final Archive base;

    private final PathMapping mapping;

    private final VirtualEntry root = new VirtualEntry();

    private VirtualEntry jcrRoot;

    public MappedArchive(Archive base, PathMapping mapping) {
        this.base = base;
        this.mapping = mapping;
    }

    @Override
    public void open(boolean strict) throws IOException {
	}

    /**
     * Decorates the non jcr files without applying the mapping. this can be done with parallel traversal.
     * @param src source entry parent
     * @param dst destination entry parent
     */
    private void applyMapping(@Nonnull Entry src, @Nonnull VirtualEntry dst) {
	}

    /**
     * Decorates the jcr entries while applying the path mapping. this cannot be done with parallel traversal, because
     * the remapping could create holes in the entry tree.
     *
     * @param src the source entry
     * @param jcrPath the jcr path of the source entry
     */
    private void applyMapping(@Nonnull Entry src, @Nonnull String jcrPath) {
	}

    @Override
    @CheckForNull
    public InputStream openInputStream(@Nullable Entry entry) throws IOException {
		return null;
	}

    @Override
    @CheckForNull
    public VaultInputSource getInputSource(@Nullable Entry entry) throws IOException {
		return null;
	}

    @Override
    @Nonnull
    public Entry getRoot() throws IOException {
		return null;
	}

    @Override
    public Entry getJcrRoot() throws IOException {
		return null;
	}

    @Override
    @Nonnull
    public MetaInf getMetaInf() {
		return null;
	}

    @Override
    public void close() {
	}

    /**
     * Implements a entry for this archive
     */
    private static class VirtualEntry implements Entry {

        @Nullable
        private final VirtualEntry parent;

        @Nonnull
        private final String name;

        @Nullable
        private Archive.Entry baseEntry;

        @Nullable
        private Map<String, VirtualEntry> children;

        private VirtualEntry() {
            this.parent = null;
            this.name = "";
            this.baseEntry = null;
        }

        private VirtualEntry(@Nonnull VirtualEntry parent, @Nonnull String name, @Nullable Archive.Entry baseEntry) {
            this.parent = parent;
            this.name = name;
            this.baseEntry = baseEntry;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        @Nonnull
        public String getName() {
			return null;
		}

        @Nonnull
        public String getPath() {
			return null;
		}

        @Nonnull
        private StringBuilder getPath(@Nonnull StringBuilder sb) {
			return null;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean isDirectory() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        @Nonnull
        public Collection<? extends Entry> getChildren() {
			return null;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        @Nullable
        public Entry getChild(String name) {
			return null;
		}

        /**
         * Adds a new child entry.
         * @param name name
         * @param baseEntry the base archive's entry or
         * @return the new entry
         */
        @Nonnull
        public VirtualEntry add(@Nonnull String name, @Nullable Archive.Entry baseEntry) {
			return null;
		}
    }
}