/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.packaging;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.jackrabbit.vault.fs.api.DumpContext;
import org.apache.jackrabbit.vault.fs.api.ImportMode;
import org.apache.jackrabbit.vault.fs.api.PathFilter;
import org.apache.jackrabbit.vault.fs.api.PathFilterSet;
import org.apache.jackrabbit.vault.fs.api.PathMapping;
import org.apache.jackrabbit.vault.fs.api.ProgressTrackerListener;
import org.apache.jackrabbit.vault.fs.api.WorkspaceFilter;
import org.apache.jackrabbit.vault.fs.config.ConfigurationException;
import org.apache.jackrabbit.vault.fs.config.DefaultWorkspaceFilter;
import org.apache.jackrabbit.vault.util.Text;
import org.w3c.dom.Element;

/**
 * Workspace filter wrapper that limits the filter in- or outside certain paths.
 * This is mostly used to limit the scope of mixed content packages to either application or content.
 */
public class ScopedWorkspaceFilter extends DefaultWorkspaceFilter {

    private static final String[] APP_ROOTS = {"/apps", "/libs" };

    private final DefaultWorkspaceFilter base;

    private final boolean allow;

    private final String[] roots;

    private final List<PathFilterSet> nodesFilterSets;

    private final List<PathFilterSet> propsFilterSets;

    public ScopedWorkspaceFilter(DefaultWorkspaceFilter base, boolean allow, String[] roots) {
        this.base = base;
        this.allow = allow;
        this.roots = roots;

        List<PathFilterSet> nodesFilterSets = new ArrayList<>();
        List<PathFilterSet> propsFilterSets = new ArrayList<>();
        for (PathFilterSet set: base.getFilterSets()) {
            String root = set.getRoot();
            if ("/".equals(root) || match(root)) {
                nodesFilterSets.add(set);
            }
        }
        for (PathFilterSet set: base.getPropertyFilterSets()) {
            String root = set.getRoot();
            if ("/".equals(root) || match(root)) {
                propsFilterSets.add(set);
            }
        }
        this.nodesFilterSets = Collections.unmodifiableList(nodesFilterSets);
        this.propsFilterSets = Collections.unmodifiableList(propsFilterSets);
    }

    private boolean match(String path) {
		return false;
	}

    public static ScopedWorkspaceFilter createApplicationScoped(DefaultWorkspaceFilter base) {
		return null;
	}

    public static ScopedWorkspaceFilter createContentScoped(DefaultWorkspaceFilter base) {
		return null;
	}

    @Override
    public void add(PathFilterSet set) {
	}

    @Override
    public void add(PathFilterSet nodeFilter, PathFilterSet propFilter) {
	}

    @Override
    public void addPropertyFilterSet(PathFilterSet set) {
	}

    @Override
    public List<PathFilterSet> getFilterSets() {
		return null;
	}

    @Override
    public List<PathFilterSet> getPropertyFilterSets() {
		return null;
	}

    @Override
    public PathFilterSet getCoveringFilterSet(String path) {
		return null;
	}

    @Override
    public ImportMode getImportMode(String path) {
		return null;
	}

    @Override
    public void setImportMode(ImportMode importMode) {
	}

    @Override
    public boolean contains(String path) {
		return false;
	}

    @Override
    public boolean covers(String path) {
		return false;
	}

    @Override
    public boolean isAncestor(String path) {
		return false;
	}

    @Override
    public boolean isGloballyIgnored(String path) {
		return false;
	}

    @Override
    public WorkspaceFilter translate(PathMapping mapping) {
		return null;
	}

    @Override
    public void load(File file) throws IOException, ConfigurationException {
	}

    @Override
    public InputStream getSource() {
		return null;
	}

    @Override
    public String getSourceAsString() {
		return null;
	}

    @Override
    public void load(InputStream in) throws IOException, ConfigurationException {
	}

    @Override
    protected PathFilter readFilter(Element elem) throws ConfigurationException {
		return null;
	}

    @Override
    public void dump(DumpContext ctx, boolean isLast) {
	}

    @Override
    public void resetSource() {
	}

    @Override
    public void setGlobalIgnored(PathFilter ignored) {
	}

    @Override
    public void dumpCoverage(Node rootNode, ProgressTrackerListener listener) throws RepositoryException {
	}

    @Override
    public void dumpCoverage(Session session, ProgressTrackerListener listener, boolean skipJcrContent) throws RepositoryException {
	}
}
