/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.webdav;

import org.apache.jackrabbit.webdav.property.DavPropertyNameSet;
import org.apache.jackrabbit.webdav.xml.DomUtil;
import org.apache.jackrabbit.webdav.xml.ElementIterator;
import org.apache.jackrabbit.webdav.xml.XmlSerializable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * MultiStatus representing the content of a multistatus response body and
 * allows to retrieve the Xml representation.
 */
public class MultiStatus implements DavConstants, XmlSerializable {

    /**
     * Map collecting the responses for this multistatus, where every href must
     * only occur one single time.
     */
    private Map<String, MultiStatusResponse> responses = new LinkedHashMap<String, MultiStatusResponse>();

    /**
     * A general response description at the multistatus top level is used to
     * provide a general message describing the overarching nature of the response.
     * If this value is available an application may use it instead of
     * presenting the individual response descriptions contained within the
     * responses.
     */
    private String responseDescription;

    /**
     * Add response(s) to this multistatus, in order to build a multistatus for
     * responding to a PROPFIND request.
     *
     * @param resource The resource to add property from
     * @param propNameSet The requested property names of the PROPFIND request
     * @param propFindType
     * @param depth
     */
    public void addResourceProperties(DavResource resource, DavPropertyNameSet propNameSet,
                                      int propFindType, int depth) {
									}

    /**
     * Add response(s) to this multistatus, in order to build a multistatus e.g.
     * in order to respond to a PROPFIND request. Please note, that in terms
     * of PROPFIND, this method would correspond to a
     * {@link DavConstants#PROPFIND_BY_PROPERTY} propfind type.
     *
     * @param resource The resource to add property from
     * @param propNameSet The requested property names of the PROPFIND request
     * @param depth
     * @see #addResourceProperties(DavResource, DavPropertyNameSet, int, int) for
     * the corresponding method that allows to specify the type.
     */
    public void addResourceProperties(DavResource resource, DavPropertyNameSet propNameSet,
                                      int depth) {
									}

    /**
     * Add response(s) to this multistatus, in order to build a multistatus
     * as returned for COPY, MOVE, LOCK or DELETE requests resulting in an error
     * with a resource other than the resource identified in the Request-URI.
     *
     * @param resource
     * @param status
     * @param depth
     */
    public void addResourceStatus(DavResource resource, int status, int depth) {
	}

    /**
     * Add a <code>MultiStatusResponse</code> element to this <code>MultiStatus</code>
     * <p>
     * This method is synchronized to avoid the problem described in
     * <a href="https://issues.apache.org/jira/browse/JCR-2755">JCR-2755</a>.
     *
     * @param response
     */
    public synchronized void addResponse(MultiStatusResponse response) {
	}

    /**
     * Returns the multistatus responses present as array.
     * <p>
     * This method is synchronized to avoid the problem described in
     * <a href="https://issues.apache.org/jira/browse/JCR-2755">JCR-2755</a>.
     *
     * @return array of all {@link MultiStatusResponse responses} present in this
     * multistatus.
     */
    public synchronized MultiStatusResponse[] getResponses() {
		return null;
	}

    /**
     * Set the response description.
     *
     * @param responseDescription
     */
    public void setResponseDescription(String responseDescription) {
	}

    /**
     * Returns the response description.
     *
     * @return responseDescription
     */
    public String getResponseDescription() {
		return null;
	}

    /**
     * Return the Xml representation of this <code>MultiStatus</code>.
     *
     * @return Xml document
     * @param document
     */
    public Element toXml(Document document) {
		return null;
	}

    /**
     * Build a <code>MultiStatus</code> from the specified xml element.
     *
     * @param multistatusElement
     * @return new <code>MultiStatus</code> instance.
     * @throws IllegalArgumentException if the given document is <code>null</code>
     * or does not provide the required element.
     */
    public static MultiStatus createFromXml(Element multistatusElement) {
		return null;
	}
}
