/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.discovery.commons.providers.base;

import java.util.LinkedList;
import java.util.List;

import org.apache.sling.discovery.TopologyEvent;
import org.apache.sling.discovery.TopologyEventListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/** 
 * SLING-4755 : background runnable that takes care of asynchronously sending events.
 * <p>
 * API is: enqueue() puts a listener-event tuple onto the internal Q, which
 * is processed in a loop in run that does so (uninterruptably, even catching
 * Throwables to be 'very safe', but sleeps 5sec if an Error happens) until
 * flushThenStop() is called - which puts the sender in a state where any pending
 * events are still sent (flush) but then stops automatically. The argument of
 * using flush before stop is that the event was originally meant to be sent
 * before the bundle was stopped - thus just because the bundle is stopped
 * doesn't undo the event and it still has to be sent. That obviously can
 * mean that listeners can receive a topology event after deactivate. But I
 * guess that was already the case before the change to become asynchronous.
 */
final class AsyncEventSender implements Runnable {
    
    static final Logger logger = LoggerFactory.getLogger(AsyncEventSender.class);

    /** stopped is always false until flushThenStop is called **/
    private boolean stopped = false;

    /** eventQ contains all AsyncEvent objects that have yet to be sent - in order to be sent **/
    private final List<AsyncEvent> eventQ = new LinkedList<AsyncEvent>();
    
    /** flag to track whether or not an event is currently being sent (but already taken off the Q **/
    private boolean isSending = false;
    
    /** Enqueues a particular event for asynchronous sending to a particular listener **/
    void enqueue(TopologyEventListener listener, TopologyEvent event) {
	}

    /** Enqueues an AsyncEvent for later in-order execution **/
    void enqueue(final AsyncEvent asyncEvent) {
	}
    
    /**
     * Stops the AsyncEventSender as soon as the queue is empty
     */
    void flushThenStop() {
	}
    
    /** Main worker loop that dequeues from the eventQ and calls sendTopologyEvent with each **/
    public void run() {
	}

    /** for testing only: checks whether there are any events being queued or sent **/
    boolean hasInFlightEvent() {
		return false;
	}

    public int getInFlightEventCnt() {
		return 0;
	}
    
}