/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.discovery.commons.providers.spi.base;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.sling.api.resource.LoginException;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceResolverFactory;
import org.apache.sling.discovery.ClusterView;
import org.apache.sling.discovery.InstanceDescription;
import org.apache.sling.discovery.commons.providers.BaseTopologyView;
import org.apache.sling.discovery.commons.providers.spi.ClusterSyncService;
import org.apache.sling.settings.SlingSettingsService;
import org.osgi.framework.Constants;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferencePolicyOption;

/**
 * The OakBacklogClusterSyncService will wait until all instances
 * in the local cluster are no longer in any backlog state.
 */
@Component(service = { ClusterSyncService.class, OakBacklogClusterSyncService.class },
            property = {
                    Constants.SERVICE_VENDOR + "=The Apache Software Foundation"
            })
public class OakBacklogClusterSyncService extends AbstractServiceWithBackgroundCheck implements ClusterSyncService {

    static enum BacklogStatus {
        UNDEFINED /* when there was an error retrieving the backlog status with oak */,
        HAS_BACKLOG /* when oak's discovery lite descriptor indicated that there is still some backlog */,
        NO_BACKLOG /* when oak's discovery lite descriptor declared we're backlog-free now */
    }

    @Reference(policyOption=ReferencePolicyOption.GREEDY)
    private IdMapService idMapService;

    @Reference(policyOption=ReferencePolicyOption.GREEDY)
    protected DiscoveryLiteConfig commonsConfig;

    @Reference(policyOption=ReferencePolicyOption.GREEDY)
    protected ResourceResolverFactory resourceResolverFactory;

    @Reference(policyOption=ReferencePolicyOption.GREEDY)
    protected SlingSettingsService settingsService;

    private ClusterSyncHistory consistencyHistory = new ClusterSyncHistory();

    public static OakBacklogClusterSyncService testConstructorAndActivate(
            final DiscoveryLiteConfig commonsConfig,
            final IdMapService idMapService,
            final SlingSettingsService settingsService,
            ResourceResolverFactory resourceResolverFactory) {
				return null;
			}

    /**
     * for testing only!
     */
    public static OakBacklogClusterSyncService testConstructor(
            final DiscoveryLiteConfig commonsConfig,
            final IdMapService idMapService,
            final SlingSettingsService settingsService,
            ResourceResolverFactory resourceResolverFactory) {
				return null;
			}

    @Activate
    protected void activate() {
	}

    public void setConsistencyHistory(ClusterSyncHistory consistencyHistory) {
	}

    public ClusterSyncHistory getConsistencyHistory() {
		return null;
	}

    /** Get or create a ResourceResolver **/
    protected ResourceResolver getResourceResolver() throws LoginException {
		return null;
	}

    @Override
    public void cancelSync() {
	}

    @Override
    public void sync(final BaseTopologyView view, final Runnable callback) {
	}

    private void waitWhileBacklog(final BaseTopologyView view, final Runnable runnable) {
	}

    private BacklogStatus getBacklogStatus(BaseTopologyView view) {
		return null;
	}

    protected DiscoveryLiteConfig getCommonsConfig() {
		return null;
	}

    protected SlingSettingsService getSettingsService() {
		return null;
	}

    public List<String> getSyncHistory() {
		return null;
	}

}
