/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to You under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apache.sling.provisioning.model;

import static org.apache.sling.provisioning.model.ModelResolveUtility.getProcessedConfiguration;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Utility for merging two models.
 *
 * @since 1.4
 */
public abstract class MergeUtility {

    /**
     * Options for specifying some parts of the merge operation.
     */
    public static class MergeOptions {

        private boolean handleRemoveRunMode = true;

        private boolean latestArtifactWins = true;

        /**
         * Returns {@code true} if the remove run mode should be respected.
         * @return {@code true} or {@code false}
         */
        public boolean isHandleRemoveRunMode() {
			return false;
		}

        /**
         * Set to {@code true} if the remove run mode should be respected.
         * @param handleRemoveRunMode Whether the remove run mode should be respected.
         * @return This instance.
         */
        public MergeOptions setHandleRemoveRunMode(boolean handleRemoveRunMode) {
			return null;
		}

        /**
         * Returns {@code true} if the latest artifact should win on a merge.
         * @return {@code true} or {@code false} if the artifact with the
         *         highest version should win
         */
        public boolean isLatestArtifactWins() {
			return false;
		}

        /**
         * Set to {@code true} if the latest artifact should win on a merge.
         * Set to {@code false} if the artifact with the highest version should win
         * @param latestArtifactWins Whether the latest artifact should win
         * @return This instance.
         */
        public MergeOptions setLatestArtifactWins(boolean latestArtifactWins) {
			return null;
		}
    }

    /**
     * Merge the additional model into the base model.
     * @param base The base model.
     * @param additional The additional model.
     */
    public static void merge(final Model base, final Model additional) {
	}

    /**
     * Merge the additional model into the base model.
     * <p>
     * Merging is performed feature by feature. Each feature is treated separately.
     * If the base model does not have a feature from the additional model, the complete
     * feature is added. If the base model has a feature which is not in the additional model,
     * the feature is left as is.
     * <p>
     * For each feature, the following actions are performed:
     * <ul>
     *   <li>If either the base feature or the additional feature has a version, then
     *    the one with the higher version is used, the other one is skipped. A missing
     *    version is considered the lowest possible version.</li>
     *   <li>The feature type of the base feature is set to the type of the additional feature.</li>
     *   <li>All additional sections of the additional feature are added to the base feature.</li>
     *   <li>All variables from the additional feature are set on the base feature, overriding
     *    values if already present.</li>
     *   <li>Each run mode of the additional feature is merged into the base feature.</li>
     * </ul>
     * <p>
     * @param base The base model.
     * @param additional The additional model.
     * @param options The merge options
     */
    public static void merge(final Model base, final Model additional, final MergeOptions options) {
	}

    /**
     * Handle the remove run mode
     * @param baseFeature The base feature
     * @param runMode The current run mode
     * @return {@code true} if the current run mode is a remove run mode
     */
    private static boolean handleRemoveRunMode(final Feature baseFeature, final RunMode runMode) {
		return false;
	}

    /**
     * Merge two configurations
     * @param baseConfig The base configuration.
     * @param mergeConfig The merge configuration.
     */
    private static void mergeConfiguration(final Configuration baseConfig, final Configuration mergeConfig) {
	}

    private static void clearConfiguration(final Configuration cfg) {
	}

    private static void copyConfigurationProperties(final Configuration baseConfig, final Configuration mergeConfig) {
	}

    /**
     * Merge the comments
     * @param base The base model object
     * @param additional The additional model object
     * @since 1.9.0
     */
    public static void mergeComments(final Commentable base, final Commentable additional) {
	}
}
